#!/usr/bin/env python
''' Olivier Morin's program (2014-09-11) to interact with FPGA  for APD acquisition. '''

import numpy
import time
from main import fpgalink2 as fpgalink2

def flReadChannel2(handle, timeout, chan, count = 1):
    ''' Read one or multiple bytes from FPGA fx2 USB interface.
        Returns the bytes. '''
    error = fpgalink2.ErrorString()
    if ( count == 1 ):
        # Read a single byte
        buf = fpgalink2.uint8()
        status = fpgalink2.fpgalink.flReadChannel(handle, timeout, chan, 1, fpgalink2.byref(buf), fpgalink2.byref(error))
        returnValue = buf.value
    else:
        # Read multiple bytes
        tmp_array = numpy.zeros(count,dtype=fpgalink2.uint8)
        byteArray = bytearray(count)
        BufType = fpgalink2.uint8*count
        buf = BufType.from_buffer(tmp_array)
        status = fpgalink2.fpgalink.flReadChannel(handle, timeout, chan, count, buf, fpgalink2.byref(error))
        # manipulation de donnees un peu laborieuse
        # a faire a l'exterieur de cette fonction peut-etre?
        tmp_array = tmp_array[::-1]
        tmp_array = numpy.unpackbits(tmp_array)
        tmp_array = tmp_array[::-1]
        array = numpy.array(tmp_array,dtype=bool)
        returnValue = tmp_array
    if ( status != fpgalink2.FL_SUCCESS ):
        s = str(error.value)
        fpgalink2.fpgalink.flFreeError(error)
        raise fpgalink2.FLException(s)
    return returnValue


class FPGA_acq:
    """ The main python class for APD acquisition FPGA
    """
    title = 'DLCZ with nanofiber'
    
    def __init__(self):
        ''' To acquire data form the fpga:
                1. call armed()
                2. call wait() until it returns True
                3. if wait() returned True, call getData
                '''

        fpgaboard_id='fffe:0002'    #original id was 1443:0007
        self.chan_dat_1 = 0xF       #Fpga address for data from APD1
        self.chan_dat_2 = 0xF+1     #idem for APD2
        self.chan_dat_3 = 0xF+2     #idem for APD3
        self.chan_int_mutex = 0xF+3
        self.chan_mutex = 0xF+4
        self.chan_trigg_cnt = 0x14

        print "Attempting to open connection to FPGALink device ..."

        self.handle = fpgalink2.flOpen(fpgaboard_id)

        print "is FPGA running:", fpgalink2.flIsFPGARunning(self.handle)

        self.num_bytes=12500 #i.e. 100kbits
        self.read_time_out=1000

        isCommCapable = fpgalink2.flIsCommCapable(self.handle)
        if isCommCapable==False:
            print('Fpga not comm capable. You probably forgot to load the program.')
            raise error
        print "is comm capable:", isCommCapable

    def getData(self,channel):
        return flReadChannel2(self.handle, self.read_time_out, channel,self.num_bytes)

    def wait(self,time_out=1000):
        for i in range(0,time_out):
            int_mutex = flReadChannel2(self.handle, 1000, self.chan_int_mutex)
            mutex = flReadChannel2(self.handle, 1000, self.chan_mutex)
            if (int_mutex!=mutex):
                #print "data ready i = ",i
                return True

        print "timout no data"
        return False

    def armed(self):
        int_mutex = flReadChannel2(self.handle, 1000, self.chan_int_mutex)
        fpgalink2.flWriteChannel(self.handle, 1000, self.chan_mutex, int_mutex)
        #print "armed (int_mutex", int_mutex
        
    def __del__(self):
        fpgalink2.flClose(self.handle)
        print "close fpga"

if __name__ == "__main__":
    ''' Example: intantiates FPGA_acq, first acquires a single set of data, then acquires 100 sets of data, and counts the number of sets where a pulse was detected from APD1. '''
    fpga_acq = FPGA_acq()
    start = time.clock()

    trigg_cnt = flReadChannel2(fpga_acq.handle, 1000, 0x14)
    print "trigg_cnt = ", trigg_cnt

    start = time.clock()
    
    fpga_acq.armed()
    
    if fpga_acq.wait():
        print('data ready')

        ans1 = fpga_acq.getData(fpga_acq.chan_dat_1)
    
        ans2 = fpga_acq.getData(fpga_acq.chan_dat_2)
    
        ans3 = fpga_acq.getData(fpga_acq.chan_dat_3)
    
        end = time.clock()
        print "time = ", end - start
        print numpy.where(ans1==1)
        print numpy.where(ans2==1)
        print numpy.where(ans3==1)
    
    cnt=0
    Ncycle = 100;
    for k in range(0,Ncycle):
        fpga_acq.armed()
        if fpga_acq.wait():
            ans1 = fpga_acq.getData(fpga_acq.chan_dat_1)
            ans2 = fpga_acq.getData(fpga_acq.chan_dat_2)
            ans3 = fpga_acq.getData(fpga_acq.chan_dat_3)
            #print numpy.where(ans1==1)
            t=numpy.where(ans1==1)[0]
            if (t.size!=0):
                cnt = cnt+1
            trigg_cnt = flReadChannel2(fpga_acq.handle, 1000, fpga_acq.chan_trigg_cnt)
            print "trigg_cnt = ", trigg_cnt           

    print "success = ", cnt*1.0/Ncycle*100,"%"
    
