''' Olivier Morin and Baptiste Gouraud's code for acquisition for APD when running the nanofiber experiment in EIT/quantum memory configuration.

This code was first implemented for acquisition with LeCroy oscilloscope then with Nexys FPGA board.

Optionally, the program can take control over the experiment in order to change automatically different parameters (exp_ctrl=True)
'''
#Note: a SEGMENT coresponds to a single write/read pulse, there are N_seq segments in a waveform acquired (one MOT cycle) and N_seg=N_seq*(number of MOT cycles) segments in a set of waveforms.

####   NOTES using LeCroy scope ####

# IP for the scope used is in setup.py

# the problem of the rescaling has been fixed by using re_init_plot.
# it was due to the axvline... however, its behaviour is not really clear
# and probably the found solution is not very "clean" (i.e. memory leaks?)

# the Run function doesn't block the UI thanks to the wx.Yield() function
# however the responsivness is not wonderful and the other strategy would
# be to use threads see http://wiki.wxpython.org/LongRunningTasks

# when using the sequence mode of the scope the exact number of data
# point can vary for instance at 4ns resolution for 1ms we have 250002 points
# for this reason when one reads the second sequence it should be shift exactly
# of this amount otherwise a shift is accumulate
# or the other solution is to manage that with the information contain in the wave desc

##################
import ctypes
from ctypes import *

import os
import pprint
import random
import sys
try:
    import wx
    import wx.py.crust as pyshell
except ImportError:
	raise ImportError,"The wxPython module is necessary to run this program."

import time
from datetime import datetime

from time import sleep

#import FPGA acquisition interface
import APDacq as fpga_acq

'''
# import LeCroy scope interface
import Lecrunch
from Lecrunch import sock
from Lecrunch import lecroy
from Lecrunch import setup
'''
import ConfigParser

import matplotlib
matplotlib.use('WXAgg')
from matplotlib.figure import Figure
from matplotlib.backends.backend_wxagg import \
    FigureCanvasWxAgg as FigCanvas, \
    NavigationToolbar2WxAgg as NavigationToolbar
import numpy as np
import pylab

import random

def wfGen(res,t1,period,choice):
    ''' Generate a waveform similar to that of an oscilloscope/FPGA for simulation purpose.
        Parameters: time resolution (ns), period = time length of a segment (ns), choice: ref,abs,etc, t1 time (ns) of the pulse in a segment '''
    N_seq = 1000
    seg_size = period/res
    elec_pulse_size = 15    #pulse amplitude (V?)
    light_pulse_size = 160  #pulse length 
    wf1 = np.array([])
    if choice==0:
        maxx=18
    elif choice==1:
        maxx=0.2
    elif choice==2:
        maxx=1
    elif choice ==3:
        maxx=15
    else:
        maxx=5
    
    for i in range(0,N_seq):
        seg1 = np.zeros(shape=(seg_size),dtype=np.int)
        seg2 = np.zeros(shape=(seg_size),dtype=np.int)
        dt = np.ceil(random.uniform(0,light_pulse_size/res))
        if random.uniform(-9,maxx)>0:
            seg1[t1/res+dt:t1/res+dt+elec_pulse_size/res]=10

        #if random.uniform(-1,2)>0:
        #    seg1[200/res+dt:200/res+dt+elec_pulse_size/res]=10

        if 100<i<300 and random.uniform(-1,2)>0: #deuniformise distribution
            seg1=np.zeros(len(seg1))

        wf1 = np.append(wf1,seg1)

    return wf1
    
class MEM_nano_fiber(wx.Frame):
    """ main class (main window)
    """
    title = 'APD signal acquistion for EIT and memory experiments with nanofiber'
    
    def __init__(self,acq_mode,exp_ctrl=False,synccfgfile=None):
        wx.Frame.__init__(self, None, -1, self.title)
        self.exp_ctrl=exp_ctrl
        self.synccfgfile=synccfgfile
        if exp_ctrl: #take the experiment control
            sys.path.append(os.path.abspath(os.path.dirname(__file__) + '/' + '../..'))
            from chaudFroid import ChaudFroid
            self.chaudFroid=ChaudFroid(waveplate=0,aramis_phaselock=1,probe45_scan=1,muwave=0,probetisaph_scan=1)
            if synccfgfile==None:
                self.chaudFroid.synchronizer.loadfromfile('configs/current.cfg')
                self.synccfgfile='configs/current.cfg'
            else:
                self.chaudFroid.synchronizer.loadfromfile(synccfgfile)

        self.cntRAM=0
        self.N_seq=0

        self.acq_mode=acq_mode
        self.Run = False    # is the acquisition running?
        
        self.save_dir = "C:\\Users\\TirageDeFibres\\Data\\Nano_Fiber_EIT_MEM\\"

        self.color_dic={}
        self.color_dic["ref"]=(1,0,0,1)
        self.color_dic["abs"]=(0,1,0,1)
        self.color_dic["back"]=(0,0,1,1)
        self.color_dic["trans"]=(0,1,1,1)
        self.color_dic["mem"]=(1,0,1,1)
        
        self.res = 10 # time resolution in nanoseconds
        self.period = 1000 # period of the read/write cycle in nanoseconds
        self.seg_size = self.period/self.res # number of point in a segment

        #### Create histograms arrays ####
        # time cursors for the two histogram, ta and tb, 1 for reference, 2 for memory readout, back from control contamination
        self.t1a = 90.0
        self.t1b = 160.0
        self.t2a = 300.0
        self.t2b = 380.0
        self.ta_back = 310.0
        self.tb_back = 360.0

        # initialization of hitograms data array

        self.hist_x_back = (np.zeros((self.seg_size,), dtype=np.float))
        self.hist_x_abs = (np.zeros((self.seg_size,), dtype=np.float))
        self.hist_x_ref = (np.zeros((self.seg_size,), dtype=np.float))
        self.hist_x_trans = (np.zeros((self.seg_size,), dtype=np.float))
        self.hist_x_mem = (np.zeros((self.seg_size,), dtype=np.float))

        self.hist_ref = (np.zeros((self.seg_size,), dtype=np.int))
        self.hist_back = (np.zeros((self.seg_size,), dtype=np.int))
        self.hist_trans = (np.zeros((self.seg_size,), dtype=np.int))
        self.hist_abs = (np.zeros((self.seg_size,), dtype=np.int))
        self.hist_mem = (np.zeros((self.seg_size,), dtype=np.int))

        self.x_vs_pulses = (np.zeros((self.seg_size,), dtype=np.float))
        self.y_vs_pulses = (np.zeros((self.seg_size,), dtype=np.float))

        #################################
        
        self.create_menu()
        self.create_main_panel()
        
        self.Load_config()        

        if self.acq_mode=="lecroy":
            self.scope=lecroy.LeCroyScope(setup.scope_ip)

        elif self.acq_mode=="fpga":
            self.Fpga_Acq = fpga_acq.FPGA_acq()

    #### Create widgets utils. ####
    def create_menu(self):
        self.menubar = wx.MenuBar()
        
        menu_file = wx.Menu()
        m_expt = menu_file.Append(-1, "&Save plot\tCtrl-S", "Save plot to file")
        self.Bind(wx.EVT_MENU, self.on_save_plot, m_expt)
        m_expt2 = menu_file.Append(-1, "&Save config.", "Save config.")
        self.Bind(wx.EVT_MENU, self.on_save_cfg, m_expt2)
        m_expt3 = menu_file.Append(-1, "&Load config.", "Load config.")
        self.Bind(wx.EVT_MENU, self.on_load_cfg, m_expt3)
        menu_file.AppendSeparator()
        m_exit = menu_file.Append(-1, "E&xit\tCtrl-X", "Exit")
        self.Bind(wx.EVT_MENU, self.on_exit, m_exit)
        self.Bind(wx.EVT_CLOSE, self.on_exit)
                
        self.menubar.Append(menu_file, "&File")
        self.SetMenuBar(self.menubar)

    def create_main_panel(self):
        self.panel = wx.Panel(self)
                
        ##  BOX FOR INPUT PARAMETERS  ##
        
        self.param_box_text = wx.StaticBox(self.panel, -1, "Parameters")
        self.param_box = wx.StaticBoxSizer(self.param_box_text, wx.VERTICAL)

        self.period_box = wx.BoxSizer(wx.HORIZONTAL)
        self.text_period = wx.StaticText(self.panel, -1,label="period read/write (ns) = ")
        self.spbutton_period = wx.SpinCtrlDouble(self.panel, value=str(self.period), size=(70,-1),
                                         min=0.0, max=1000000, inc=100)
        self.Bind(wx.EVT_SPINCTRLDOUBLE, self.update_time_param, self.spbutton_period)
        self.period_box.Add(self.text_period, 0, wx.ALL, 0)
        self.period_box.Add(self.spbutton_period, 0, wx.ALL, 0)

        #### t1 ####
        self.time1_box = wx.BoxSizer(wx.HORIZONTAL)

        self.text_t1a = wx.StaticText(self.panel, -1,label="ta ref (ns) = ")
        self.spbutton_t1a = wx.SpinCtrlDouble(self.panel, value=str(self.t1a), size=(60,-1),
                                         min=0.0, max=1000000, inc=0.25)
        self.Bind(wx.EVT_SPINCTRLDOUBLE, self.update_time_param, self.spbutton_t1a)
        self.text_t1b = wx.StaticText(self.panel, -1,label="tb ref (ns) = ")
        self.spbutton_t1b = wx.SpinCtrlDouble(self.panel, value=str(self.t1b), size=(60,-1),
                                         min=0.0, max=1000000, inc=0.25)
        self.Bind(wx.EVT_SPINCTRLDOUBLE, self.update_time_param, self.spbutton_t1b)

        self.time1_box.Add(self.text_t1a, 0, wx.ALL, 0)
        self.time1_box.Add(self.spbutton_t1a, 0, wx.ALL, 0)
        self.time1_box.AddSpacer(20)
        self.time1_box.Add(self.text_t1b, 0, wx.ALL, 0)
        self.time1_box.Add(self.spbutton_t1b, 0, wx.ALL, 0)

        #### t2 ####
        self.time2_box = wx.BoxSizer(wx.HORIZONTAL)

        self.text_t2a = wx.StaticText(self.panel, -1,label="ta mem (ns) = ")
        self.spbutton_t2a = wx.SpinCtrlDouble(self.panel, value=str(self.t2a), size=(60,-1),
                                         min=0.0, max=1000000, inc=0.25)
        self.Bind(wx.EVT_SPINCTRLDOUBLE, self.update_time_param, self.spbutton_t2a)
        self.text_t2b = wx.StaticText(self.panel, -1,label="tb mem (ns) = ")
        self.spbutton_t2b = wx.SpinCtrlDouble(self.panel, value=str(self.t2b), size=(60,-1),
                                         min=0.0, max=1000000, inc=0.25)
        self.Bind(wx.EVT_SPINCTRLDOUBLE, self.update_time_param, self.spbutton_t2b)

        self.time2_box.Add(self.text_t2a, 0, wx.ALL, 0)
        self.time2_box.Add(self.spbutton_t2a, 0, wx.ALL, 0)
        self.time2_box.AddSpacer(20)
        self.time2_box.Add(self.text_t2b, 0, wx.ALL, 0)
        self.time2_box.Add(self.spbutton_t2b, 0, wx.ALL, 0)

        #### t_back ####
        self.timeback_box = wx.BoxSizer(wx.HORIZONTAL)

        self.text_ta_back = wx.StaticText(self.panel, -1,label="ta back (ns) = ")
        self.spbutton_ta_back = wx.SpinCtrlDouble(self.panel, value=str(self.ta_back), size=(60,-1),
                                         min=0.0, max=1000000, inc=0.25)
        self.Bind(wx.EVT_SPINCTRLDOUBLE, self.update_time_param, self.spbutton_ta_back)
        self.text_tb_back = wx.StaticText(self.panel, -1,label="tb back (ns) = ")
        self.spbutton_tb_back = wx.SpinCtrlDouble(self.panel, value=str(self.tb_back), size=(60,-1),
                                         min=0.0, max=1000000, inc=0.25)
        self.Bind(wx.EVT_SPINCTRLDOUBLE, self.update_time_param, self.spbutton_tb_back)

        self.timeback_box.Add(self.text_ta_back, 0, wx.ALL, 0)
        self.timeback_box.Add(self.spbutton_ta_back, 0, wx.ALL, 0)
        self.timeback_box.AddSpacer(20)
        self.timeback_box.Add(self.text_tb_back, 0, wx.ALL, 0)
        self.timeback_box.Add(self.spbutton_tb_back, 0, wx.ALL, 0)

        ####
        self.param_box.Add(self.period_box, 0, wx.ALL, 5)
        self.param_box.Add(self.time1_box, 0, wx.ALL, 5)
        self.param_box.Add(self.time2_box, 0, wx.ALL, 5)
        self.param_box.Add(self.timeback_box, 0, wx.ALL, 5)

        ## BOX FOR OUTPUT RESULTS (basic data analysis)##

        self.N_seg_ref=0
        self.N_seg_back=0
        self.N_seg_abs=0
        self.N_seg_trans=0
        self.N_seg_mem=0

        self.n_ref=0.0
        self.n_back=0.0
        self.n_abs=0.0
        self.n_trans=0.0
        self.n_mem=0.0
        self.n_backonly=0.0
        self.n_leak=0.0

        self.N_ref=0
        self.N_back=0
        self.N_abs=0
        self.N_trans=0
        self.N_mem=0
        self.N_backonly=0.0
        self.N_leak=0.0

        self.OD = -1.0
        self.Trans = -1.0
        self.Mem = -1.0
       
        self.N_seg = 0
        
        self.results_box_text = wx.StaticBox(self.panel, -1, "Results")
        self.results_box = wx.StaticBoxSizer(self.results_box_text, wx.VERTICAL)
        
        self.grid_results = wx.FlexGridSizer( cols=2 )
        

        self.text_n_ref = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="n_ref = "+str(self.n_ref))
        self.text_n_ref.SetForegroundColour(tuple(i * 255 for i in self.color_dic["ref"]))
        self.text_N_ref = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="("+str(self.N_ref)+"/"+str(self.N_seg_ref)+")")

        self.text_n_ref.SetToolTip(wx.ToolTip("n_ : mean number of photon between the cursors\n"
                                                "N_ : number of detection between the cursors\n"
                                                "n_seg_ : number of trials\n"
                                                "n_ =   (N_/N_seg)"))
        
        self.text_n_back = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="n_back = "+str(self.n_back))
        self.text_n_back.SetForegroundColour(tuple(i * 255 for i in self.color_dic["mem"]))
        self.text_N_back = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="("+str(self.N_back)+"/"+str(self.N_seg_mem)+")")
        
        self.text_n_abs = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="n_abs = "+str(self.n_abs))
        self.text_n_abs.SetForegroundColour(tuple(i * 255 for i in self.color_dic["abs"]))
        self.text_N_abs = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="("+str(self.N_abs)+"/"+str(self.N_seg_abs)+")")
        
        self.text_n_trans = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="n_trans = "+str(self.n_trans))
        self.text_n_trans.SetForegroundColour(tuple(i * 255 for i in self.color_dic["trans"]))
        self.text_N_trans = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="("+str(self.N_trans)+"/"+str(self.N_seg_trans)+")")

        self.text_n_mem = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="n_mem = "+str(self.n_mem))
        self.text_n_mem.SetForegroundColour(tuple(i * 255 for i in self.color_dic["mem"]))
        self.text_N_mem = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="("+str(self.N_mem)+"/"+str(self.N_seg_mem)+")")

        self.text_n_leak = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="n_leak = "+str(self.n_leak))
        self.text_n_leak.SetForegroundColour(tuple(i * 255 for i in self.color_dic["mem"]))
        self.text_N_leak = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="("+str(self.N_leak)+"/"+str(self.N_seg_mem)+")")

        self.text_n_backonly = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="n_ctrlonly = "+str(self.n_backonly))
        self.text_n_backonly.SetForegroundColour(tuple(i * 255 for i in self.color_dic["back"]))
        self.text_N_backonly = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="("+str(self.N_backonly)+"/"+str(self.N_seg_back)+")")

        self.grid_results.Add( self.text_n_ref,   0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_N_ref,   0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_n_abs,   0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_N_abs,   0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_n_leak, 0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_N_leak, 0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_n_mem,   0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_N_mem,   0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_n_back,  0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_N_back,  0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_n_backonly, 0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_N_backonly, 0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_n_trans, 0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )
        self.grid_results.Add( self.text_N_trans, 0, wx.ALIGN_CENTRE|wx.LEFT|wx.RIGHT|wx.TOP, 5 )

        self.results_box.Add( self.grid_results, 0, wx.ALIGN_CENTRE|wx.ALL, 5 )

        self.text_OD = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="Optical Density = "+"%.2f" % self.OD)
        self.text_OD.SetToolTip(wx.ToolTip("OD = -ln(n_abs/n_ref)"))
        self.results_box.Add( self.text_OD, 0, wx.ALIGN_CENTRE|wx.ALL, 5 )

        self.text_Trans = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="Transparency = "+"%.2f" % self.Trans)
        self.text_Trans.SetToolTip(wx.ToolTip("Trans = (n_trans-n_abs-n*_back)/n_ref"))
        self.results_box.Add( self.text_Trans, 0, wx.ALIGN_CENTRE|wx.ALL, 5 )

        self.text_Mem = wx.StaticText(self.panel, -1,size=(100,-1),
                                      label="Memory = "+"%.4f" % self.Mem)
        self.text_Mem.SetToolTip(wx.ToolTip("Memory = (n_mem-n**_back)/n_ref"))
        self.results_box.Add( self.text_Mem, 0, wx.ALIGN_CENTRE|wx.ALL, 5 )

        ## BOX FOR RUN ##
        
        self.run_box_text = wx.StaticBox(self.panel, -1, "Run")
        self.run_box = wx.StaticBoxSizer(self.run_box_text, wx.HORIZONTAL)
        self.sampleList = ['Reference','Absorption','Memory','ControlOnly','Transparency']
        self.run_choice = wx.RadioBox(
                self.panel, -1, "", wx.DefaultPosition, wx.DefaultSize,
                self.sampleList, 1, wx.RA_SPECIFY_COLS | wx.NO_BORDER
                )
        self.scan_choices = wx.CheckListBox(
                self.panel, -1, wx.DefaultPosition, wx.DefaultSize,
                choices=self.sampleList, style=wx.RA_SPECIFY_COLS | wx.NO_BORDER
                )
        
        #self.Bind(wx.EVT_RADIOBOX, self.EvtRadioBox, rb)
        self.run_choice.SetToolTip(wx.ToolTip("choose the type of data that is going to be measured"))
        
        self.run_button = wx.ToggleButton(self.panel, -1, "Run/Stop")
        self.run_button.SetValue(False)
        self.Bind(wx.EVT_TOGGLEBUTTON, self.on_Run_button, self.run_button)

        self.runRAM_button = wx.ToggleButton(self.panel, -1, "Run +scan config")
        self.runRAM_button.SetValue(False)
        self.Bind(wx.EVT_TOGGLEBUTTON, self.on_RunRAM_button, self.runRAM_button)

        self.runCoh_button = wx.ToggleButton(self.panel, -1, "Run +scan param")
        self.runCoh_button.SetValue(False)
        self.Bind(wx.EVT_TOGGLEBUTTON, self.on_RunCoh_button, self.runCoh_button)

        self.ctrl_exp_checkbx = wx.CheckBox(self.panel, -1, "Auto Control")
        self.text_cycles = wx.StaticText(self.panel, -1,label="Acq cycles: ")
        self.spCtrl_cycles = wx.SpinCtrl(self.panel, initial=100,value=str(100), size=(70,-1),
                                         min=0, max=1000000000)
        self.text_tCoh = wx.StaticText(self.panel, -1,label="param (min,max,step): ")
        self.spCtrl_tCohmin = wx.SpinCtrl(self.panel, initial=50,value=str(50), size=(70,-1),
                                         min=-10000000, max=1000000)
        self.spCtrl_tCohmax = wx.SpinCtrl(self.panel, initial=100,value=str(100), size=(70,-1),
                                         min=-100000000, max=1000000)
        self.spCtrl_tCohstep = wx.SpinCtrl(self.panel, initial=10,value=str(10), size=(70,-1),
                                         min=0, max=1000000)
        self.scantypes=["t_","tsub","polar","phaselockfreq","VCOfreqDiode","VCOfreqTisaph","MWdetuningkHz","dummy"]   
        self.t__choices={
            "t0":0,"t1":1,"t2":2,"t3":3,"t4":4,"t5":5,"t6":6,"t7":7,"t8":8,"t9":9,
            "tA":10,"tB":11,"tC":12,"tD":13,"tE":14,"tF":15,"tG":16,"tH":17,"tI":18,"tJ":19,"tK":20,"tL":21,"tM":22,"tN":23,"tO":24,"tP":25,"tQ":26,"tR":27,"tS":28,"tT":29,"tU":30,"tV":31
            }  
        self.scantype_choice = wx.Choice(self.panel, choices=self.scantypes)
        self.txtparam_tCoh = wx.Choice(self.panel, choices=sorted(self.t__choices.keys()))
        self.text_units = wx.StaticText(self.panel, -1,label="units: kHz, deg, 10ns")
        self.scanwindow_checkbx = wx.CheckBox(self.panel, -1, "scan tmem window")

        self.run_options_box = wx.BoxSizer(wx.VERTICAL)
        self.cycles_box = wx.BoxSizer(wx.HORIZONTAL)
        
        self.cycles_box.Add(self.text_cycles, 0, wx.ALL, 0)
        self.cycles_box.Add(self.spCtrl_cycles, 0, wx.ALL, 0)
        self.run_options_box.Add(self.run_button, 0, wx.ALL, 0)
        self.run_options_box.AddSpacer(10)
        self.run_options_box.Add(self.run_choice, 0, wx.ALL, 0)
        self.run_options_box.AddSpacer(10)
        self.run_options_box.Add(self.ctrl_exp_checkbx, 0, wx.ALL, 0)
        self.run_options_box.AddSpacer(10)
        self.run_options_box.Add(self.cycles_box, 0, wx.ALL, 0)

        self.run_scan_box = wx.BoxSizer(wx.VERTICAL)
        self.run_scan_box.Add(self.runRAM_button, 0, wx.ALL, 0)
        self.run_scan_box.Add(self.scan_choices, 0, wx.ALL, 0)

        self.run_tcoh_box = wx.BoxSizer(wx.VERTICAL)
        self.run_tcoh_box.Add(self.runCoh_button, 0, wx.EXPAND|wx.RIGHT|wx.BOTTOM, 3)
        self.run_tcoh_box.Add(self.text_tCoh, 0, wx.ALL, 0)
        self.run_tcoh_box.Add(self.spCtrl_tCohmin, 0, wx.ALL, 0)
        self.run_tcoh_box.Add(self.spCtrl_tCohmax, 0, wx.ALL, 0)
        self.run_tcoh_box.Add(self.spCtrl_tCohstep, 0, wx.ALL, 0)
        self.run_tcoh_box.Add(self.scantype_choice, 0, wx.ALL, 0)
        self.run_tcoh_box.Add(self.text_units, 0, wx.ALL, 0)
        self.run_tcoh_box.Add(self.txtparam_tCoh, 0, wx.ALL, 0)
        self.run_tcoh_box.Add(self.scanwindow_checkbx, 0, wx.ALL, 0)
        
        self.run_box.Add(self.run_options_box, 0, wx.EXPAND|wx.RIGHT|wx.BOTTOM, 3)
        self.run_box.AddSpacer(10)
        self.run_box.Add(self.run_scan_box, 0, wx.EXPAND|wx.RIGHT|wx.BOTTOM, 3)
        self.run_box.AddSpacer(10)
        self.run_box.Add(self.run_tcoh_box, 0, wx.EXPAND|wx.RIGHT|wx.BOTTOM, 3)
        
        ## BOX FOR SAVING TASKS ##

        self.save_box_text = wx.StaticBox(self.panel, -1, "Save")
        self.save_box = wx.StaticBoxSizer(self.save_box_text, wx.VERTICAL)
        self.save_checkbox = wx.BoxSizer(wx.HORIZONTAL)
        
        self.save_wf_check = wx.CheckBox(self.panel, -1, " waveforms")
        self.save_plot_check = wx.CheckBox(self.panel, -1, " plot")
        self.save_trigger_check = wx.CheckBox(self.panel, -1, " triggers")
        self.save_report_check = wx.CheckBox(self.panel, -1, " report")
        self.comments_stat = wx.StaticText(self.panel, -1, "Comments")
        self.comments_ctrl = wx.TextCtrl(self.panel, -1,
                        "",
                       size=(300,100), style=wx.TE_MULTILINE|wx.TE_PROCESS_ENTER)
        self.save_dir_bt = wx.Button(self.panel,50,"save dir.")#
        self.Bind(wx.EVT_BUTTON, self.on_save_dir, self.save_dir_bt)

        self.load_synccfg_bt = wx.Button(self.panel,50,"Load fpga SyncCfg")#
        self.Bind(wx.EVT_BUTTON, self.on_load_synccfg_bt, self.load_synccfg_bt)
        

        self.save_checkbox.Add(self.save_wf_check, 0, wx.ALL, 0)
        self.save_checkbox.AddSpacer(4)
        self.save_checkbox.Add(self.save_plot_check, 0, wx.ALL, 0)
        self.save_checkbox.AddSpacer(4)
        self.save_checkbox.Add(self.save_trigger_check, 0, wx.ALL, 0)
        self.save_checkbox.AddSpacer(4)
        self.save_checkbox.Add(self.save_report_check, 0, wx.ALL, 0)
        self.save_box.Add(self.save_checkbox, 0, wx.ALL, 0)
        self.save_box.AddSpacer(4)
        self.save_box.Add(self.comments_stat, 0, wx.ALL, 0)
        self.save_box.Add(self.comments_ctrl, 0, wx.ALL, 0)
        self.save_box.Add(self.save_dir_bt, 0, wx.ALL, 0)
        self.save_box.Add(self.load_synccfg_bt, 0, wx.ALL, 0)

        
        ## PLOT DISPLAY BOX ##
        self.plot_box_text = wx.StaticBox(self.panel, -1, "Plot display")
        self.plot_box = wx.StaticBoxSizer(self.plot_box_text, wx.VERTICAL)
        self.plot_ref_check = wx.CheckBox(self.panel, -1, " ref")
        self.plot_abs_check = wx.CheckBox(self.panel, -1, " abs")
        self.plot_back_check = wx.CheckBox(self.panel, -1, " ctrl")
        self.plot_trans_check = wx.CheckBox(self.panel, -1, " trans")
        self.plot_mem_check = wx.CheckBox(self.panel, -1, " mem")
        self.Bind(wx.EVT_CHECKBOX, self.on_cb_plot_ref_check, self.plot_ref_check)
        self.Bind(wx.EVT_CHECKBOX, self.on_cb_plot_abs_check, self.plot_abs_check)
        self.Bind(wx.EVT_CHECKBOX, self.on_cb_plot_back_check, self.plot_back_check)
        self.Bind(wx.EVT_CHECKBOX, self.on_cb_plot_trans_check, self.plot_trans_check)
        self.Bind(wx.EVT_CHECKBOX, self.on_cb_plot_mem_check, self.plot_mem_check)
        self.plot_ref_check.SetForegroundColour(tuple(i * 255 for i in self.color_dic["ref"]))
        self.plot_abs_check.SetForegroundColour(tuple(i * 255 for i in self.color_dic["abs"]))
        self.plot_back_check.SetForegroundColour(tuple(i * 255 for i in self.color_dic["back"]))
        self.plot_trans_check.SetForegroundColour(tuple(i * 255 for i in self.color_dic["trans"]))
        self.plot_mem_check.SetForegroundColour(tuple(i * 255 for i in self.color_dic["mem"]))
        self.plot_ref_check.SetValue(True)
        self.plot_abs_check.SetValue(True)
        self.plot_back_check.SetValue(True)
        self.plot_trans_check.SetValue(True)
        self.plot_mem_check.SetValue(True)
        self.plot_box.Add(self.plot_ref_check, 0, wx.ALL, 0)
        self.plot_box.AddSpacer(4)
        self.plot_box.Add(self.plot_abs_check, 0, wx.ALL, 0)
        self.plot_box.AddSpacer(4)
        self.plot_box.Add(self.plot_mem_check, 0, wx.ALL, 0)
        self.plot_box.AddSpacer(4)
        self.plot_box.Add(self.plot_back_check, 0, wx.ALL, 0)
        self.plot_box.AddSpacer(4)
        self.plot_box.Add(self.plot_trans_check, 0, wx.ALL, 0)
    
        self.dpi = 100        
        self.fig, (self.axes) = pylab.subplots(1, sharex=True)
        self.fig_vs_pulses, (self.ax_vs_pulses) = pylab.subplots(1, sharex=True)
        self.fig_coherence, (self.ax_coherence) = pylab.subplots(1, sharex=True)
        self.init_plot()
        #self.init_plot_vs_pulse()
       
        self.nb = wx.Notebook(self.panel)

        self.canvas = FigCanvas(self.nb, -1, self.fig)
        self.canvas_vs_pulses = FigCanvas(self.nb, -1, self.fig_vs_pulses)
        self.canvas_coherence = FigCanvas(self.nb, -1, self.fig_coherence)

        ### Command line interface
        if self.exp_ctrl:
            CF=self.chaudFroid
        self.shell = pyshell.Crust(self.panel,wx.ID_ANY,locals=locals())
 
        ##### General layout
        self.hbox1 = wx.BoxSizer(wx.HORIZONTAL)

        self.hbox2 = wx.BoxSizer(wx.VERTICAL)

        self.hbox2.Add(self.param_box, border=5, flag=wx.ALL)
        self.hbox2.AddSpacer(4)
        
        self.hbox2.Add(self.results_box, border=5, flag=wx.ALL)
        self.hbox2.AddSpacer(4)
        
        self.hbox2.Add(self.save_box, border=5, flag=wx.ALL)

        self.hbox2.AddSpacer(4)
        
        self.hbox1.Add(self.run_box, border=5, flag=wx.ALL)
        self.hbox1.Add(self.plot_box, border=5, flag=wx.ALL)

        
        # add the pages to the notebook with the label to show on the tab
        self.nb.AddPage(self.canvas, "Main")
        self.nb.AddPage(self.canvas_vs_pulses, "vs pulse")
        self.nb.AddPage(self.canvas_coherence, "Coherence time")

        
        self.vbox = wx.BoxSizer(wx.VERTICAL)
        self.vbox.Add(self.shell, 0, wx.GROW)
        self.vbox.Add(self.nb, 0, flag=wx.LEFT | wx.TOP | wx.GROW)        
        self.vbox.Add(self.hbox1, 0, flag=wx.ALIGN_LEFT | wx.TOP)

        self.mainbox = wx.BoxSizer(wx.HORIZONTAL)
        self.mainbox.Add(self.vbox, 2, flag=wx.LEFT | wx.TOP | wx.GROW)        
        self.mainbox.Add(self.hbox2, 1, flag=wx.ALL | wx.ALIGN_CENTER_HORIZONTAL)        
        
        self.panel.SetSizer(self.mainbox)
        self.vbox.Fit(self)
        self.mainbox.Fit(self)

    #############################
    #### Tools for plotting. ####
    #############################
    def init_plot(self):
        self.axes.cla()
        
        self.ax_vs_pulses.cla()
        
        self.axes.set_axis_bgcolor('black')
        self.ax_vs_pulses.set_axis_bgcolor('black')

        self.axes.set_xlabel("time (ns)")
        self.axes.set_ylabel("density of events (events/ns)")
        self.axes.grid(True, color='gray')
        
        self.vline_t1a = self.axes.axvline(self.t1a, color = 'r')
        self.vline_t1b = self.axes.axvline(self.t1b, color = 'r')
        self.vline_t2a = self.axes.axvline(self.t2a, color = 'b')
        self.vline_t2b = self.axes.axvline(self.t2b, color = 'b')
        self.vline_ta_back = self.axes.axvline(self.ta_back, color = 'g')
        self.vline_tb_back = self.axes.axvline(self.tb_back, color = 'g')
        

        # plot the data as a line series, and save the reference 
        # to the plotted line series
        #
        if self.plot_ref_check.IsChecked():
            self.plot_data_ref = self.axes.plot(
                np.array(self.hist_x_ref),self.hist_ref,
                linewidth=1,
                color = self.color_dic["ref"]
                )[0]
        if self.plot_back_check.IsChecked():
            self.plot_data_back = self.axes.plot(
                np.array(self.hist_x_back),self.hist_back,
                linewidth=1,
                color = self.color_dic["back"]
                )[0]
        if self.plot_abs_check.IsChecked():
            self.plot_data_abs = self.axes.plot(
                np.array(self.hist_x_abs),self.hist_abs,
                linewidth=1,
                color = self.color_dic["abs"]
                )[0]
        if self.plot_trans_check.IsChecked():
            self.plot_data_trans = self.axes.plot(
                np.array(self.hist_x_trans),self.hist_trans,
                linewidth=1,
                color = self.color_dic["trans"]
                )[0]
        if self.plot_mem_check.IsChecked():
            self.plot_data_mem = self.axes.plot(
                np.array(self.hist_x_mem),self.hist_mem,
                linewidth=1,
                color = self.color_dic["mem"]
                )[0]
        '''
        self.plot_data_vs_pulses = self.ax_vs_pulses.bar(
            np.array(self.x_vs_pulses),
            self.y_vs_pulses,
            linewidth=1,
            color=(1, 1, 0),
            )[0]    
        '''
        self.plot_data_vs_pulses = self.ax_vs_pulses.plot(
            np.array(self.x_vs_pulses),
            self.y_vs_pulses,
            linewidth=1,
            color=(1, 1, 0),
            )[0]  

    def set_data(self,choice):
        ''' Put the last data acquired in the right choice. '''
        if self.sampleList[choice]=='Reference':
            self.hist_ref = self.hist_Y
            self.hist_x_ref = self.hist_X
            self.N_seg_ref=self.N_seg
        elif self.sampleList[choice]=='ControlOnly':
            self.hist_back = self.hist_Y
            self.hist_x_back = self.hist_X
            self.N_seg_back=self.N_seg
        elif self.sampleList[choice]=='Absorption':
            self.hist_abs = self.hist_Y
            self.hist_x_abs = self.hist_X
            self.N_seg_abs=self.N_seg
        elif self.sampleList[choice]=='Transparency':
            self.hist_trans = self.hist_Y
            self.hist_x_trans = self.hist_X
            self.N_seg_trans=self.N_seg
        elif self.sampleList[choice]=='Memory':
            self.hist_mem = self.hist_Y
            self.hist_x_mem = self.hist_X
            self.N_seg_mem=self.N_seg

    def draw_plot(self):
        """ Redraws the plot
        """
        self.plot_data_ref.set_xdata(self.hist_x_ref)
        if self.N_seg_ref!=0:
            self.plot_data_ref.set_ydata(self.hist_ref*1.0/(self.res*self.N_seg_ref))
        self.plot_data_back.set_xdata(self.hist_x_back)
        if self.N_seg_back!=0:
            self.plot_data_back.set_ydata(self.hist_back*1.0/(self.res*self.N_seg_back))
        self.plot_data_abs.set_xdata(self.hist_x_abs)
        if self.N_seg_abs!=0:
            self.plot_data_abs.set_ydata(self.hist_abs*1.0/(self.res*self.N_seg_abs))
        self.plot_data_trans.set_xdata(self.hist_x_trans)
        if self.N_seg_trans!=0:
            self.plot_data_trans.set_ydata(self.hist_trans*1.0/(self.res*self.N_seg_trans))
        self.plot_data_mem.set_xdata(self.hist_x_mem)
        if self.N_seg_mem!=0:
            self.plot_data_mem.set_ydata(self.hist_mem*1.0/(self.res*self.N_seg_mem))

        #!#
        self.plot_data_vs_pulses.set_xdata(np.array(self.x_vs_pulses))
        if self.N_seg!=0: # if usefull here?
            self.plot_data_vs_pulses.set_ydata(self.y_vs_pulses)
        #!#
                
        self.axes.relim()
        self.ax_vs_pulses.relim()
        self.axes.autoscale_view()
        self.ax_vs_pulses.autoscale_view()
                   
        self.canvas.draw()
        self.canvas_vs_pulses.draw()


    def on_cb_plot_ref_check(self,event):

        if self.plot_ref_check.IsChecked():
            self.plot_data_ref = self.axes.plot(
                np.array(self.hist_x_ref),self.hist_ref,
                linewidth=2,
                color = self.color_dic["ref"]
                )[0]
        else:
            self.plot_data_ref.remove()
            
        self.draw_plot()

    def on_cb_plot_abs_check(self,event):

        if self.plot_abs_check.IsChecked():
            self.plot_data_abs = self.axes.plot(
                np.array(self.hist_x_abs),self.hist_abs,
                linewidth=2,
                color = self.color_dic["abs"]
                )[0]
        else:
            self.plot_data_abs.remove()
            
        self.draw_plot()
        
    def on_cb_plot_back_check(self,event):

        if self.plot_back_check.IsChecked():
            self.plot_data_back = self.axes.plot(
                np.array(self.hist_x_back),self.hist_back,
                linewidth=2,
                color = self.color_dic["back"]
                )[0]
        else:
            self.plot_data_back.remove()
            
        self.draw_plot()
        
    def on_cb_plot_trans_check(self,event):

        if self.plot_trans_check.IsChecked():
            self.plot_data_trans = self.axes.plot(
                np.array(self.hist_x_trans),self.hist_trans,
                linewidth=2,
                color = self.color_dic["trans"]
                )[0]
        else:
            self.plot_data_trans.remove()
            
        self.draw_plot()


    def on_cb_plot_mem_check(self,event):

        if self.plot_mem_check.IsChecked():
            self.plot_data_mem = self.axes.plot(
                np.array(self.hist_x_mem),self.hist_mem,
                linewidth=2,
                color = self.color_dic["mem"]
                )[0]
        else:
            self.plot_data_mem.remove()
            
        self.draw_plot()


    ##############################
    # Handle basic GUI's events
    ###############################
    
    def on_save_plot(self,event):
        file_choices = "PNG (*.png)|*.png"
        
        dlg = wx.FileDialog(
            self, 
            message="Save plot as...",
            defaultDir=os.getcwd(),
            defaultFile="plot.png",
            wildcard=file_choices,
            style=wx.SAVE)
        
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            self.canvas.print_figure(path, dpi=self.dpi)
            #self.flash_status_message("Saved to %s" % path)

    def on_save_dir(self,event):
        file_choices = "PNG (*.png)|*.png"
        
        dlg = wx.DirDialog(
            self, 
            message="Save in directory...",
            #defaultDir=os.getcwd(),
            #defaultFile="plot.png",
            #wildcard=file_choices,
            style=wx.DD_DEFAULT_STYLE | wx.DD_DIR_MUST_EXIST)
        
        if dlg.ShowModal() == wx.ID_OK:
            self.save_dir = dlg.GetPath()
            print(self.save_dir)
            self.save_dir_bt.SetToolTip(wx.ToolTip(self.save_dir))
            #self.canvas.print_figure(path, dpi=self.dpi)
            #self.flash_status_message("Saved to %s" % path)
 
    def on_save_cfg(self,event):
        file_choices = "CFG (*.cfg)|*.cfg"
        
        dlg = wx.FileDialog(
            self, 
            message="Save cfg as...",
            defaultDir=os.getcwd(),
            defaultFile="DLCZ_0.cfg",
            wildcard=file_choices,
            style=wx.SAVE)
        
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            self.Save_config(path)
            #self.canvas.print_figure(path, dpi=self.dpi)
            #self.flash_status_message("Saved to %s" % path)


    def on_load_cfg(self,event):
        file_choices = "CFG (*.cfg)|*.cfg"
        
        dlg = wx.FileDialog(
            self, 
            message="Load cfg from...",
            defaultDir=os.getcwd(),
            defaultFile="*.cfg",
            wildcard=file_choices,
            style=wx.OPEN)
        
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            self.Load_config(path)
            #self.canvas.print_figure(path, dpi=self.dpi)
            #self.flash_status_message("Saved to %s" % path)
       
    def on_exit(self, event):
        self.Run = False
        self.Save_config()
        pylab.close(self.fig) #otherwise the process is still running
        pylab.close(self.fig_vs_pulses) #otherwise the process is still running
        pylab.close(self.fig_coherence)
        self.Destroy()

    def on_load_synccfg_bt(self,event):
        if self.exp_ctrl:
            file_choices = "CFG (*.cfg)|*.cfg"
        
            dlg = wx.FileDialog(
                self, 
                message="Load FPGA synchronizer config file...",
                defaultDir='configs',
                defaultFile="current.cfg",
                wildcard=file_choices,
                style=wx.OPEN)
        
            if dlg.ShowModal() == wx.ID_OK:
                path = dlg.GetPath()
                self.chaudFroid.synchronizer.loadfromfile(path)
                self.synccfgfile=path
        else:
            print("No control on the experiment: restart the program with exp_ctrl=True if required.")
            

    ################################
    #### Some update functions. ####
    def update_results(self):
        
        self.text_n_ref.SetLabel("n_ref = "+"%.5f" % self.n_ref)
        self.text_N_ref.SetLabel("("+str(self.N_ref)+"/"+str(self.N_seg_ref)+")")

        self.text_n_abs.SetLabel("n_abs = "+"%.5f" % self.n_abs)
        self.text_N_abs.SetLabel("("+str(self.N_abs)+"/"+str(self.N_seg_abs)+")")

        self.text_n_back.SetLabel("n_back = "+"%.5f" % self.n_back)
        self.text_N_back.SetLabel("("+str(self.N_back)+"/"+str(self.N_seg_mem)+")")

        self.text_n_trans.SetLabel("n_trans = "+"%.5f" % self.n_trans)
        self.text_N_trans.SetLabel("("+str(self.N_trans)+"/"+str(self.N_seg_trans)+")")

        self.text_n_mem.SetLabel("n_mem = "+"%.5f" % self.n_mem)
        self.text_N_mem.SetLabel("("+str(self.N_mem)+"/"+str(self.N_seg_mem)+")")

        self.text_n_leak.SetLabel("n_leak = "+"%.5f" % self.n_leak)
        self.text_N_leak.SetLabel("("+str(self.N_leak)+"/"+str(self.N_seg_mem)+")")

        self.text_n_backonly.SetLabel("n_ctrlonly = "+"%.5f" % self.n_backonly)
        self.text_N_backonly.SetLabel("("+str(self.N_backonly)+"/"+str(self.N_seg_back)+")")

        self.text_OD.SetLabel("Optical Density = "+"%.2f" % self.OD)
        self.text_Trans.SetLabel("Transparency = "+"%.2f" % (self.Trans*100) + " %")
        
        self.text_Mem.SetLabel("Memory = "+"%.2f" % (self.Mem*100) + " %")
        
    def update_time_param(self,event):
        self.t1a =self.spbutton_t1a.GetValue()
        self.t1b =self.spbutton_t1b.GetValue()
        self.t2a =self.spbutton_t2a.GetValue()
        self.t2b =self.spbutton_t2b.GetValue()
        self.ta_back =self.spbutton_ta_back.GetValue()
        self.tb_back =self.spbutton_tb_back.GetValue()
        self.period =self.spbutton_period.GetValue()
        self.vline_t1a.set_xdata(self.t1a)
        self.vline_t1b.set_xdata(self.t1b)
        self.vline_t2a.set_xdata(self.t2a)
        self.vline_t2b.set_xdata(self.t2b)
        self.vline_ta_back.set_xdata(self.ta_back)
        self.vline_tb_back.set_xdata(self.tb_back)
        self.Save_config()

        self.set_results()
        self.update_results()
        self.draw_plot()  

    #################################
    #### Loading and saving routines.
    #################################
    def Save_config(self,path="defMEM.cfg",opt_dict=None):
        Config = ConfigParser.ConfigParser()
        Config.add_section('def')

        # time param
        Config.set('def','t1a', self.t1a)
        Config.set('def','t1b', self.t1b)
        Config.set('def','t2a', self.t2a)
        Config.set('def','t2b', self.t2b)
        Config.set('def','ta_back', self.ta_back)
        Config.set('def','tb_back', self.tb_back)
        Config.set('def','period', self.period)

        # save dir
        Config.set('def','save_dir', self.save_dir)

        # things to save
        Config.set('def','save_wf',self.save_wf_check.GetValue())
        Config.set('def','save_plot',self.save_plot_check.GetValue())
        Config.set('def','save_trigger',self.save_trigger_check.GetValue())
        Config.set('def','save_report',self.save_report_check.GetValue())

        Config.set('def','auto_ctrl',self.ctrl_exp_checkbx.GetValue())
        Config.set('def','cycles',self.spCtrl_cycles.GetValue())

        Config.set('def','comments',self.comments_ctrl.GetValue().encode('utf-8'))
        Config.set('def','N_seq',self.N_seq)

        Config.set('def','tCohmin',self.spCtrl_tCohmin.GetValue())
        Config.set('def','tCohmax',self.spCtrl_tCohmax.GetValue())
        Config.set('def','tCohstep',self.spCtrl_tCohstep.GetValue())

        Config.set('def','scantype_choice',self.scantype_choice.GetSelection())
        Config.set('def','txtparam_tCoh',self.txtparam_tCoh.GetSelection())
        Config.set('def','scanwindow_checkbx',self.scanwindow_checkbx.GetValue())

        #TODO next line not working yet
        #Config.set('def','scan_choices',self.scan_choices.GetChecked())

        Config.set('def','synccfg',self.synccfgfile)

        if opt_dict != None:
            for k,v in opt_dict.items():
                Config.set('def',k,v)

        cfgfile = open(path,'w')
        Config.write(cfgfile)
        cfgfile.close()

    def Load_config(self,path="defMEM.cfg"):
        Config = ConfigParser.ConfigParser()
        Config.read(path)

        # time param
        self.t1a = Config.getfloat('def','t1a')
        self.t1b = Config.getfloat('def','t1b')
        self.t2a = Config.getfloat('def','t2a')
        self.t2b = Config.getfloat('def','t2b')
        self.ta_back = Config.getfloat('def','ta_back')
        self.tb_back = Config.getfloat('def','tb_back')
        self.period = Config.getfloat('def','period')
        #display
        self.spbutton_t1a.SetValue(self.t1a)
        self.spbutton_t1b.SetValue(self.t1b)
        self.spbutton_t2a.SetValue(self.t2a)
        self.spbutton_t2b.SetValue(self.t2b)
        self.spbutton_ta_back.SetValue(self.ta_back)
        self.spbutton_tb_back.SetValue(self.tb_back)
        self.spbutton_period.SetValue(self.period)
        self.update_time_param(self)

        #save dir
        self.save_dir = Config.get('def','save_dir')
        self.save_dir_bt.SetToolTip(wx.ToolTip(self.save_dir))

        # things to save
        self.save_wf_check.SetValue(Config.getboolean('def','save_wf'))
        self.save_plot_check.SetValue(Config.getboolean('def','save_plot'))
        self.save_trigger_check.SetValue(Config.getboolean('def','save_trigger'))
        self.save_report_check.SetValue(Config.getboolean('def','save_report'))
        if not self.save_trigger_check.GetValue():
            print("Warning, not saving raw data!!!")

        self.ctrl_exp_checkbx.SetValue(Config.getboolean('def','auto_ctrl'))
        self.spCtrl_cycles.SetValue(Config.getint('def','cycles'))

        self.spCtrl_tCohmin.SetValue(Config.getint('def','tCohmin'))
        self.spCtrl_tCohmax.SetValue(Config.getint('def','tCohmax'))
        self.spCtrl_tCohstep.SetValue(Config.getint('def','tCohstep'))

        self.scantype_choice.SetSelection(Config.getint('def','scantype_choice'))
        self.txtparam_tCoh.SetSelection(Config.getint('def','txtparam_tCoh'))
        self.scanwindow_checkbx.SetValue(Config.getboolean('def','scanwindow_checkbx'))

        #TODO next line not working yet
        #self.scan_choices.SetChecked(map(int,Config.get('def','scan_choices')[1:-1].split(',')))
        
    def Write_report(self,start_date,stop_date,choice):
        #the_date = datetime.today()

        run_dir = self.save_dir+start_date.strftime("\\20%y-%m-%d")
        if not os.path.exists(run_dir):
            os.makedirs(run_dir)
        path = run_dir + '\\Nanof_EIT_MEM_report.txt'
        file = open(path,"a")
        file.write(start_date.strftime("\n\n%y-%m-%d // %H:%M:%S - "))
        file.write(stop_date.strftime("%H:%M:%S\n"))
        file.write("=========================\n")
        
        file.write("measurement: "+self.sampleList[choice]+"\n")

        file.write("n_ref = "+"%.5f" % self.n_ref)
        file.write("   ("+str(self.N_ref)+"/"+str(self.N_seg_ref)+")"+"\n")

        file.write("n_abs = "+"%.5f" % self.n_abs)
        file.write("   ("+str(self.N_abs)+"/"+str(self.N_seg_abs)+")"+"\n")

        file.write("n_leak = "+"%.5f" % self.n_leak)
        file.write("    ("+str(self.N_leak)+"/"+str(self.N_seg_mem)+")"+"\n")

        file.write("n_mem = "+"%.5f" % self.n_mem)
        file.write("    ("+str(self.N_mem)+"/"+str(self.N_seg_mem)+")"+"\n")

        file.write("n_back = "+"%.5f" % self.n_back)
        file.write("    ("+str(self.N_back)+"/"+str(self.N_seg_mem)+")"+"\n")

        file.write("n_ctrlonly = "+"%.5f" % self.n_backonly)
        file.write("    ("+str(self.N_backonly)+"/"+str(self.N_seg_back)+")"+"\n")

        file.write("n_trans = "+"%.5f" % self.n_trans)
        file.write("    ("+str(self.N_trans)+"/"+str(self.N_seg_trans)+")"+"\n")


        file.write("Optical Density = "+"%.2f" % self.OD+"\n")
        file.write("Transparency = "+"%.2f" % (self.Trans*100)+"%\n")
        file.write("Memory = "+"%.2f" % (self.Mem*100)+"%\n")
        
        #file.write("\n\n" + self.comments_ctrl.GetValue())
        file.write("\n\n" + self.comments_ctrl.GetValue().encode('utf-8'))

        file.write("\n--------------\n\n\n")

        file.close()

    def Save_plot(self,run_dir):
        self.canvas.print_figure(run_dir + "\plot.png", dpi=self.dpi)
        
    #################################
    #### Loading and saving routines.  --> end
    #################################

    #############################################################################
    #############################################################################
    #### RUN ACQUISITION BUTTONS
    #############################################################################
    #############################################################################    
    
    def on_Run_button(self, event):
        ''' Acquire data for the selected "choice". '''
        self.Run = self.run_button.GetValue()
        choice = self.run_choice.GetSelection()
        
        if self.Run==True:
            #wx.Yield() #commented on 2015-06-08, reason for this being here unsure for now
            self.Run_exp(choice)
        else:
            pass

    def on_RunRAM_button(self,event):
        ''' Acquire data, scanning the different selected choices. '''
        date = datetime.today()
        print(date.strftime("\\%Hh%Mm%Ss"))
        
        self.RunRAM = self.runRAM_button.GetValue()
        while self.RunRAM:        
            for cntRAM in [0,1,2,3,4]:
                if cntRAM in self.scan_choices.GetChecked():
                    self.run_button.SetValue(True)
                    self.Run=True
                    self.run_choice.SetSelection(cntRAM)
                    self.Run_exp(cntRAM,self.spCtrl_cycles.GetValue(),subdir=("\\cycle"+date.strftime("%Hh%Mm%Ss")))
                if not(self.runRAM_button.GetValue()):
                    break
                
        else:
            self.run_button.SetValue(False)
            self.Run=False            

    def on_RunCoh_button(self,event):
        ''' Acquire data, scanning for a selected parameter.'''
        mini=self.spCtrl_tCohmin.GetValue()
        maxi=self.spCtrl_tCohmax.GetValue()
        step=self.spCtrl_tCohstep.GetValue()
        paramstring=self.scantypes[self.scantype_choice.GetSelection()]

        tparam=self.txtparam_tCoh.GetSelection()
        #print(tparam)
        #tparamstring=str(tparam)
        #tparam=self.t__choices[tparamstring]
        if paramstring=="t_":
            paramstring+=str(tparam)
            
        trange=range(mini,maxi,step)
        
        if self.runCoh_button.GetValue():
            start_date = datetime.today()
            run_dir = self.save_dir+start_date.strftime("\\20%y-%m-%d")
            if not os.path.exists(run_dir):
                os.makedirs(run_dir)
            subdir=start_date.strftime("\\%Hh%Mm%Ss_scanning_")+"%s"%paramstring
            run_dir2 = run_dir+subdir
            os.makedirs(run_dir2)

            if 1 in self.scan_choices.GetChecked():
                abss=[]
            if 2 in self.scan_choices.GetChecked():         
                mems=[]
                leaks=[]
            if 4 in self.scan_choices.GetChecked():
                transs=[]
                       
            t2asav=self.t2a
            t2bsav=self.t2b
            for t in trange:
                print('%s: %d'%(paramstring,t))

                if self.exp_ctrl:
                    if paramstring[:2]=="t_":
                        self.chaudFroid.synchronizer.SyncCfg['tparams'][tparam]=t
                        self.chaudFroid.synchronizer.updateTparam(tparam)
                    elif paramstring=="tsub":
                        self.chaudFroid.synchronizer.SyncCfg['subsequence']['tsub']=t
                        self.chaudFroid.synchronizer.updateSubsequence()
                    elif paramstring=="polar":
                        self.chaudFroid.waveplate.MoveAbsoluteRot(float(t))
                        sleep(1) #might be unecessary if reference is taken for a long enough time
                    elif paramstring=="phaselockfreq":
                        self.chaudFroid.phaselock.setDetuning(t,unit='kHz')
                    elif paramstring=="VCOfreqDiode":
                        self.chaudFroid.aom.set_detuning(t/1000.)
                    elif paramstring=="VCOfreqTisaph":
                        self.chaudFroid.aomtisaph.set_detuning(t/1000.)
                    elif paramstring=="MWdetuningkHz":
                        self.chaudFroid.muwave_gen.setFrequency(t*1000+9192638000) #t is detuning in kHz
                    elif paramstring=="dummy":
                        print('dummy: doing nothing ;)')
                    ### TODO: diplay units            

                for cnt in [0,1,2,3,4]:
                    if cnt in self.scan_choices.GetChecked():
                        self.run_button.SetValue(True)
                        self.Run=True
                        self.run_choice.SetSelection(cnt)
                        self.Run_exp(cnt,self.spCtrl_cycles.GetValue(),opt_dict={paramstring:t},subdir=subdir)
                    
                    if not(self.runCoh_button.GetValue()):
                        break

                if 1 in self.scan_choices.GetChecked():
                    abss.append((self.Abs,self.Abs_err))
                if 2 in self.scan_choices.GetChecked():         
                    mems.append((self.Mem,self.Mem_err))
                    leaks.append((self.Leak,self.Leak_err))
                if 4 in self.scan_choices.GetChecked():
                    transs.append((self.Trans_noabsSub,self.Trans_noabsSub_err))               
                
                if (paramstring[:2]=="t_" and self.scanwindow_checkbx.GetValue()): 
                    self.spbutton_t2a.SetValue(self.t2a+step*10)
                    self.spbutton_t2b.SetValue(self.t2b+step*10)
                    self.update_time_param(event)

                if not(self.runCoh_button.GetValue()):
                    break

                
            if paramstring=="MWdetuningkHz":    
                self.chaudFroid.muwave_gen.setFrequency(9192638000) #uwave frequency return to resonance 


            if 2 in self.scan_choices.GetChecked():
                print("memory efficiencies")
                path = run_dir2 + '\\scan_%s_memoryefficiency.txt'%paramstring
                f = open(path,"a")
                f.write("#t value\tMemory efficiency\tError\n")
                for i,t in enumerate(trange):
                    print(str(t)+"  %.2f"%(mems[i][0]*100))
                    f.write(str(t)+"\t%.5f\t%.5f\n"%(mems[i]))
                f.close()

            if 1 in self.scan_choices.GetChecked():
                print("transmissions")
                path = run_dir2 + '\\scan_%s_transmission.txt'%paramstring
                f = open(path,"a")
                f.write("#t value\tTransmission\tError\n")
                for i,t in enumerate(trange):
                    print(str(t)+"  %.5f"%(abss[i][0]*100))
                    f.write(str(t)+"\t%.5f\t%.5f\n"%(abss[i]))
                f.close()

            if 2 in self.scan_choices.GetChecked():
                print("leaks")
                path = run_dir2 + '\\scan_%s_leak.txt'%paramstring
                f = open(path,"a")
                f.write("#t value\tLeak\tError\n")
                for i,t in enumerate(trange):
                    print(str(t)+"  %.2f"%(leaks[i][0]*100))
                    f.write(str(t)+"\t%.5f\t%.5f\n"%(leaks[i]))
                f.close()

            if 4 in self.scan_choices.GetChecked():
                print("transparencies")
                path = run_dir2 + '\\scan_%s_induced_trans.txt'%paramstring
                f = open(path,"a")
                f.write("#t value\tInducedTrans\tError\n")
                for i,t in enumerate(trange):
                    print(str(t)+"  %.5f"%(transs[i][0]*100))
                    f.write(str(t)+"\t%.5f\t%.5f\n"%(transs[i]))
                f.close()

            if self.exp_ctrl:
                if paramstring[:2]=="t_":
                    self.chaudFroid.synchronizer.SyncCfg['tparams'][tparam]=trange[0]
                    self.chaudFroid.synchronizer.updateTparam(tparam)

            self.spbutton_t2a.SetValue(t2asav)
            self.spbutton_t2b.SetValue(t2bsav)
            self.update_time_param(event)
            self.runCoh_button.SetValue(False)
        else:
            self.run_button.SetValue(False)
            self.Run=False
            
    #############################################################################
    #############################################################################
    #### END  of RUN ACQUISITION BUTTONS          END
    #############################################################################
    #############################################################################    

    #############################
    #### Some data analysis. ####
    #############################
    def N_in_time(self,Hist,ta,tb):
        ''' Counts events in histogram between times ta and tb. '''                
        valid = np.where(Hist>ta,True,False) & np.where(Hist<tb,True,False)
        return sum(Hist[int(round(ta)):int(round(tb))])

    
    def set_results(self):
        ''' Compute the different interesting values from last data. '''
        # Counts are first selected with the windows defined by ta,tb or t2a, t2b
        self.N_ref   = self.N_in_time(self.hist_ref,  self.t1a/self.res,self.t1b/self.res)
        self.N_abs   = self.N_in_time(self.hist_abs,  self.t1a/self.res,self.t1b/self.res)
        self.N_back  = self.N_in_time(self.hist_mem, self.ta_back/self.res,self.tb_back/self.res)
        #self.N_back=0
        self.N_trans = self.N_in_time(self.hist_trans,self.t1a/self.res,self.t1b/self.res)
        self.N_mem = self.N_in_time(self.hist_mem,self.t2a/self.res,self.t2b/self.res)
        self.N_leak = self.N_in_time(self.hist_mem,self.t1a/self.res,self.t1b/self.res)
        self.N_backonly = self.N_in_time(self.hist_back,self.t1a/self.res,self.t1b/self.res)
        self.N_dark = self.N_in_time(self.hist_ref,self.ta_back/self.res,self.tb_back/self.res)
        #self.N_dark=0

        #### WARNING: PUT BACK AND DARK Manually to 0
        ###############################################
        #### WARNING: PUT BACK AND DARK Manually to 0
        ##############################################
        #### WARNING: PUT BACK AND DARK Manually to 0
        ##############################################

        # Number ofdetected photons per segment (single write/read cycle)
        if self.N_seg_ref!=0:
            self.n_ref = 1.0*self.N_ref/self.N_seg_ref
            self.n_dark = 1.0*self.N_dark/self.N_seg_ref
            self.n_dark_refscaled=self.n_dark*abs(self.t1b-self.t1a)/abs(self.tb_back-self.ta_back)
        else:
            self.n_ref=-1
            self.n_dark=-1
    
        if self.N_seg_abs!=0:
            self.n_abs = 1.0*self.N_abs/self.N_seg_abs
        else:
            self.n_abs=-1
            
        if self.N_seg_mem!=0:
            self.n_mem = 1.0*self.N_mem/self.N_seg_mem
            self.n_back = 1.0*self.N_back/self.N_seg_mem
            self.n_back_refscaled = self.n_back*abs(self.t1b-self.t1a)/abs(self.tb_back-self.ta_back)
            self.n_back_memscaled = self.n_back*abs(self.t2b-self.t2a)/abs(self.tb_back-self.ta_back)
            self.n_leak = 1.0*self.N_leak/self.N_seg_mem
        else:
            self.n_mem=-1
            self.n_back=-1
            self.n_leak=-1
            
        if self.N_seg_trans!=0:
            self.n_trans = 1.0*self.N_trans/self.N_seg_trans
        else:
            self.n_trans=-1

        if self.N_seg_back!=0:
            self.n_backonly = 1.0*self.N_backonly/self.N_seg_back
        else:
            self.n_backonly=-1
            
        # OD
        if self.n_ref>0 and self.n_abs>=0:
            if self.n_dark>=0:
                self.Abs= (self.n_abs-self.n_dark_refscaled)/(self.n_ref-self.n_dark_refscaled)
                self.OD =  -np.log((self.n_abs-self.n_dark_refscaled)/(self.n_ref-self.n_dark_refscaled))
            else:
                self.Abs=self.n_abs/self.n_ref
                self.OD =  -np.log(self.n_abs/self.n_ref)
        else:
            self.Abs = -1.0
            self.OD = -1.0

        # Transparency
        if self.n_ref>0 and self.n_back>=0 and self.n_abs>=0 and self.n_trans>=0 and self.n_dark>=0:
            self.Trans = (self.n_trans-self.n_back_refscaled-self.n_abs)/(self.n_ref-self.n_dark_refscaled)
            self.Trans_noabsSub = (self.n_trans-self.n_back_refscaled)/(self.n_ref-self.n_dark_refscaled)
        elif self.n_ref>0 and self.n_abs>=0 and self.n_trans>=0 and self.n_back==-1:
            self.Trans = (self.n_trans-self.n_abs)/(self.n_ref)
            self.Trans_noabsSub = (self.n_trans)/(self.n_ref)            
        else:
            self.Trans = -1.0
            self.Trans_noabsSub= -1.0

        # Memory efficiency    
        if self.n_ref>0 and self.n_back>=0 and self.n_mem>=0 and self.n_dark<self.n_ref:
            self.Mem = (self.n_mem-self.n_back_memscaled)/(self.n_ref-self.n_dark_refscaled)
            ### TODO: a more intelligent substraction should be done for n_mem, here we substract too much
        else:
            self.Mem = -1.0

        # Leakage
        if self.n_ref>0 and self.n_backonly>=0 and self.n_leak>=0:
            self.Leak = (self.n_leak-self.n_backonly)/(self.n_ref-self.n_dark_refscaled)
        else:
            self.Leak = -1.0
            
        #########################
        #### Computing ERROR BARS
        ##########################
        if self.N_seg_ref!=0:
            self.n_ref_err = np.sqrt(1.0*self.N_ref)/self.N_seg_ref
            self.n_dark_err = np.sqrt(1.0*self.N_dark)/self.N_seg_ref
            self.n_dark_refscaled_err=self.n_dark_err*abs(self.t1b-self.t1a)/abs(self.tb_back-self.ta_back)
        else:
            self.n_ref_err=-1
            self.n_dark_err=-1
    
        if self.N_seg_abs!=0:
            self.n_abs_err = np.sqrt(1.0*self.N_abs)/self.N_seg_abs
        else:
            self.n_abs=-1
            
        if self.N_seg_mem!=0:
            self.n_mem_err = np.sqrt(1.0*self.N_mem)/self.N_seg_mem
            self.n_back_err = np.sqrt(1.0*self.N_back)/self.N_seg_mem
            self.n_back_refscaled_err = self.n_back_err*abs(self.t1b-self.t1a)/abs(self.tb_back-self.ta_back)
            self.n_back_memscaled_err = self.n_back_err*abs(self.t2b-self.t2a)/abs(self.tb_back-self.ta_back)
            self.n_leak_err = np.sqrt(1.0*self.N_leak)/self.N_seg_mem
        else:
            self.n_mem_err=-1
            self.n_back_err=-1
            self.n_leak_err=-1
            
        if self.N_seg_trans!=0:
            self.n_trans_err = np.sqrt(1.0*self.N_trans)/self.N_seg_trans
        else:
            self.n_trans=-1

        if self.N_seg_back!=0:
            self.n_backonly_err = np.sqrt(1.0*self.N_backonly)/self.N_seg_back
        else:
            self.n_backonly_err=-1

        # OD
        if self.n_ref>0 and self.n_abs>=0:
            if self.n_dark>=0:
                self.Abs_err= np.sqrt(self.n_abs_err**2 + ((self.n_abs-self.n_dark_refscaled)**2*self.n_ref_err**2 + (self.n_abs-self.n_ref)**2*self.n_dark_refscaled_err**2)/(self.n_ref-self.n_dark_refscaled)**2)/abs(self.n_ref-self.n_dark_refscaled)
            else:
                self.Abs_err= np.sqrt(self.n_abs_err**2 + (self.n_abs**2*self.n_ref_err**2)/self.n_ref**2)/abs(self.n_ref)
            self.OD_err = self.Abs_err/self.Abs
        
        else:
            self.Abs_err = -1.0
            self.OD_err = -1.0

        # Transparency
        if self.n_ref>0 and self.n_back>=0 and self.n_abs>=0 and self.n_trans>=0 and self.n_dark>=0:
            self.Trans_err = np.sqrt(self.n_trans_err**2+self.n_back_refscaled_err**2+self.n_abs_err**2+(self.n_ref_err**2+self.n_dark_refscaled_err**2)*self.Trans**2)/abs(self.n_ref-self.n_dark_refscaled)
            self.Trans_noabsSub_err = np.sqrt(self.n_trans_err**2+self.n_back_refscaled_err**2+(self.n_ref_err**2+self.n_dark_refscaled_err**2)*self.Trans_noabsSub**2)/abs(self.n_ref-self.n_dark_refscaled)
        elif self.n_ref>0 and self.n_abs>=0 and self.n_trans>=0:
            self.Trans_err = np.sqrt(self.n_trans_err**2+self.n_abs_err**2+(self.n_ref_err**2)*self.Trans**2)/abs(self.n_ref)
            self.Trans_noabsSub_err = np.sqrt(self.n_trans_err**2+(self.n_ref_err**2)*self.Trans_noabsSub**2)/abs(self.n_ref)
        else:
            self.Trans_err = -1.0
            self.Trans_noabsSub_err= -1.0

        # Memory efficiency    
        if self.n_ref>0 and self.n_back>=0 and self.n_mem>=0 and self.n_dark<self.n_ref:
            self.Mem_err = np.sqrt(self.n_mem_err**2+self.n_back_memscaled_err**2+(self.n_ref_err**2+self.n_dark_refscaled_err**2)*self.Mem**2)/abs(self.n_ref-self.n_dark_refscaled)
            ### TODO: a more intelligent substraction should be done for n_mem, here we substract too much
        else:
            self.Mem_err = -1.0

        # Leakage
        if self.n_ref>0 and self.n_backonly>=0 and self.n_leak>=0:
            self.Leak_err = np.sqrt(self.n_leak_err**2+self.n_backonly_err**2+(self.n_ref_err**2+self.n_dark_refscaled_err**2)*self.Leak**2)/abs(self.n_ref-self.n_dark_refscaled)
            (self.n_leak-self.n_backonly)/(self.n_ref-self.n_dark_refscaled)
        else:
            self.Leak_err = -1.0

    #######################################################
    #######################################################
    ####  MAIN ROUTINE:                                ####
    ####  Acquisition and preprocessing/plotting loop. ####
    #######################################################
    #######################################################
    def Run_exp(self, choice, cycles=-1,opt_dict=None,subdir=""):
        ''' Run the experiment. Choice is reference, absorption etc...'''
        if not(self.Run):
            return
        
        if (self.exp_ctrl and self.ctrl_exp_checkbx.GetValue()):
            self.chaudFroid.set_choice(self.sampleList[choice])
            sleep(1)

        # Create proper directories for data saving.
        start_date = datetime.today()
        run_dir = self.save_dir+start_date.strftime("\\20%y-%m-%d")
        if not os.path.exists(run_dir):
            os.makedirs(run_dir)
        run_dir2 = run_dir+subdir+start_date.strftime("\\%Hh%Mm%Ss")
        os.makedirs(run_dir2)
        print(run_dir2)

        # Depending on acquisition device, define:
        # self.res : time resolution, nanoseconds (typic 10ns)
        # self.period : is the time length of one segment (typic 1mus)
        # self.seg_size : is the number of points per segment
        if self.acq_mode=="lecroy":
            self.wdsc = self.scope.getwavedesc(1)

            fTreshold = 0.5
            threshold = (fTreshold+self.wdsc['vertical_offset'])/self.wdsc['vertical_gain']
        
            self.res = self.wdsc['horiz_interval']*1e9
          
        elif self.acq_mode=="sim":
            threshold = 5
            self.res = 10
       
        elif self.acq_mode=="fpga":
            self.res = 10
        
        res = self.res
        self.seg_size = int(round(self.period/self.res ))
        seg_size = self.seg_size

        # initialization of histograms data arrays
        self.hist_X = np.linspace(0,self.period-res,num=seg_size)
        self.hist_Y = (np.zeros((seg_size,), dtype=np.int))

        # Display update
        #self.set_data(choice)
        #self.set_results()
        #self.init_plot()

        self.N_seq = 0  # Number of segments (write/read cycle) per waveform (will be corrected is the following part)
        self.N_seg = 0  # Total number of segments acquired (=N_seq*number_of_waveforms_acquired)
        
        cnt=0
        
        ### loop  ###
        while self.Run: #and self.N < 5000:
            # Acquire data, wf1 is the waevform, wf1_tmp is the binary waveform.
            start = time.clock()
            start_total=start

            if self.acq_mode=='lecroy':
                wf1 = self.scope.getwaveform( 1, self.wdsc)
                wf1_tmp = np.where(wf1>threshold,1,0)

            elif self.acq_mode=='sim':
                t1=100
                wf1 = wfGen(self.res,t1,self.period,choice)
                wf1_tmp = np.where(wf1>threshold,1,0)

            elif self.acq_mode=='fpga':
                self.Fpga_Acq.armed() # envoyer armed just after get Data pour aller plus vite
                self.Fpga_Acq.wait()
                wf1=self.Fpga_Acq.getData(self.Fpga_Acq.chan_dat_1)
                #print(wf1)
                wf1_tmp=wf1
                
            else:
                print("ERROR: unknown acquisition mode")
                                
            #end = time.clock()
            #print "loading time = ", end - start

            #start = time.clock()
            N_seq = int(round(wf1.size/(self.period/res))) #Number of segments in a waveform
            self.N_seq=N_seq            
            self.N_seg += self.N_seq    #Total number of segments acquired
            
            if cnt==0: # Reinitialise x_vs_pulses/y_vs_pulses on first acquisition loop
                self.x_vs_pulses = np.linspace(1,N_seq,num=N_seq)
                self.y_vs_pulses = (np.zeros((N_seq,), dtype=np.int))

            # Detection of negative edges, trigg1 are the indexes when a negedge is detected
            trigg1 = np.where((wf1_tmp[0:N_seq*seg_size-1]-wf1_tmp[1:N_seq*seg_size])==1)[0]

            # Eventually save raw data and negative edges detected
            if(self.save_wf_check.GetValue()):
                f1 = open(run_dir2+"\\wf_"+self.sampleList[choice]+".npy", 'a+b')
                np.save(f1, wf1)
                f1.close()
                
            if(self.save_trigger_check.GetValue()):
                T1 = open(run_dir2+"\\negedge_"+self.sampleList[choice]+".npy", 'a+b')
                np.save(T1, trigg1)
                T1.close()
                        
            # list of corresponding segment number for each detected event
            trigg1_k = trigg1/seg_size # N.B. : division entiere
            # trigger time in the segment (read/write window)
            Trigg1 = trigg1%seg_size #Trigg1 is an array of indexes i where events were detected 0<=i<segment_length (write/read cycle length)
            
            self.y_vs_pulses[trigg1_k]=self.y_vs_pulses[trigg1_k]+1            
            

            # At this points Trigg1 is an array of indexes i where events were detected 0<=i<segment_length (write/read cycle length)
            hist, bin_edge = np.histogram(Trigg1,bins = range(0,seg_size+1))
            hist[0]=0
            
            self.hist_Y += hist
            #end = time.clock()
            #print "processing time = ", end - start

            start = time.clock()

            # Update plot every 20 acquisition cycle.
            if cnt%20==0:
                self.set_data(choice)   # sets the data in the right place depending on the choice
                self.set_results()      # basic analysis of data
                self.update_results()   # show results from analysis
                self.draw_plot()        # redraw

            cnt=cnt+1

            wx.Yield()
           
            #end = time.clock()
            #end_total = end
            #print "rate = ", N_seq/(end_total-start_total)

            if cnt==cycles:
                self.set_data(choice)   # sets the data in the right place depending on the choice
                self.set_results()      # basic analysis of data
                self.update_results()   # show results from analysis
                self.draw_plot()        # redraw
                self.run_button.SetValue(False)
                self.Run=False

        # When stopped, eventually update everything, save report, and plots.    
        self.set_data(choice)   # sets the data in the right place depending on the choice
        self.set_results()      # basic analysis of data
        self.update_results()   # show results from analysis
        self.draw_plot()        # redraw

        stop_date = datetime.today()
        if(self.save_report_check.GetValue()):
            self.Write_report(start_date,stop_date,choice)
        if(self.save_plot_check.GetValue()):
            self.Save_plot(run_dir2)
        self.Save_config(run_dir2+"/config.cfg",opt_dict)

if __name__ == '__main__':
    
    app = wx.App()
    cfgpath='C:\\Users\\TirageDeFibres\\GitRepos\\SoftNanof\\devices\\FpgaProgramming\\SyncConfigs\\'
    #app.frame = MEM_nano_fiber(acq_mode='sim',exp_ctrl=False)
    #app.frame = MEM_nano_fiber(acq_mode='fpga',exp_ctrl=True,synccfgfile=cfgpath'2015-02-27_APDabs45and44.cfg')
    #app.frame = MEM_nano_fiber(acq_mode='fpga',exp_ctrl=False,synccfgfile=cfgpath'2015-02-27_APDabs45and44.cfg')
    #app.frame = MEM_nano_fiber(acq_mode='fpga',exp_ctrl=True,synccfgfile=cfgpath'2015-06-10_effect_of_microwave_fiberedprobe_wdipole_APD_additionalrepump_pushoutbeam_correctednames.cfg')
    #app.frame = MEM_nano_fiber(acq_mode='fpga',exp_ctrl=True,synccfgfile=cfgpath'2015-02-17_dipoletrap_APD_45trans_correctednames.cfg')
    #app.frame = MEM_nano_fiber(acq_mode='fpga',exp_ctrl=True,synccfgfile=cfgpath+'2015-06-23_dipoletrap_APD_45trans_correctednames_longtime.cfg')
    #app.frame = MEM_nano_fiber(acq_mode='fpga',exp_ctrl=True,synccfgfile=cfgpath+'2015-06-24_effect_of_microwave_fiberedprobe_wdipole_APD_additionalrepump_pushoutbeam_45tisaph_longtime.cfg')
    app.frame = MEM_nano_fiber(acq_mode='fpga',exp_ctrl=True,synccfgfile=cfgpath+'2015-07-01_EIT_molasses.cfg')
    #app.frame = MEM_nano_fiber(acq_mode='fpga',exp_ctrl=True,synccfgfile=cfgpath'configs/2015-07-01_EIT_molasses_44&45_APD.cfg')
    #app.frame = MEM_nano_fiber(acq_mode='fpga',exp_ctrl=True,synccfgfile=cfgpath'configs/2015-07-02_newEIT3_molasses_APD_AramisSingle_Signal.cfg')
    
    
    app.frame.Show()
    app.MainLoop()
