#!/usr/bin/env python
"""
Different utilities which should be independent of the particular Fpga circuit design used.
Includes compilation (veri2bit), and configuration loading (bit2Fpga).
See the last, if __name__ =="__main__":, section for usage examples.
"""
import os, sys

fpgalinkpath=os.path.join(os.path.dirname(os.path.realpath(__file__)),'libfpgalink-20120621')
pypath=os.path.join(fpgalinkpath,'examples','python')
sys.path.insert(0, pypath)

if sys.platform=='linux2':
    ldlibpath=os.path.join(fpgalinkpath,'linux.x86_64','rel')
    updatedLD=False
    if 'LD_LIBRARY_PATH' not in os.environ:
        os.environ['LD_LIBRARY_PATH'] = ldlibpath
        updatedLD=True
    if ldlibpath not in os.environ['LD_LIBRARY_PATH']:
        os.environ['LD_LIBRARY_PATH'] = ldlibpath+":"+os.environ['LD_LIBRARY_PATH']
        updatedLD=True
    if updatedLD==True:
        try:
            os.execv(sys.argv[0], sys.argv)
        except Exception, exc:
            print 'Failed re-exec, make sure you have permission to execute the file:', exc
            sys.exit(1)

import fpgalink2

# Import file manipulation tools.
import shutil
import glob

import sys
import subprocess

chipname='xc6slx16-3-csg324' # Fpga chip name
originalVp = "1443:0007" # Vendor and product IDs for the Fpga board. Here the Nexys3 board ids.
syncVp = "fffe:0001" # Because two boards with same IDs are used in our lab, we give different IDs to the synchronizer and the acquisition device.
APDacqVp = "fffe:0002"

### Below is defined the mapping between the final user inputs/outputs (left side), and the pins of the FPGA chip (right side).
### Should be adjusted using the documentation of your FPGA chip and board (i.e. example Nexys3_master.ucf file provided by Digilent).
###
### In our case the user inouts are dispatched from the VHDC connector to BNC connectors labeled 'DIO.., AO., AI.,' for salvaging reasons even if they are all used as digital inputs/outputs.
### The LEDi outputs are the debugging LEDs of the Nexys3 Board.
bnc2ucf = {
        'LED0':'U16',
        'LED1':'V16',
        'LED2':'U15',
        'LED3':'V15',
        'LED4':'M11',
        'LED5':'N11',
        'LED6':'R11',
        'LED7':'T11',
        'DIO0':'B2',
        'DIO1':'A2',
        'DIO2':'D6',
        'DIO3':'C6',
        'DIO4':'B3',
        'DIO5':'A3',
        'DIO6':'B4',
        'DIO7':'A4',
        'AO0':'C5',
        'AO1':'A5',
        'AO2':'B6',
        'AO3':'A6',
        'AO4':'C7',
        'AO5':'A7',
        'AO6':'D8',
        'AO7':'C8',
        'AI0+':'A16',
        'AI0-':'B16',
        'AI1+':'C14',
        'AI1-':'D14',
        'AI2+':'A15',
        'AI2-':'C15',
        'AI3+':'E13',
        'AI3-':'F13',
        'AI4+':'A14',
        'AI4-':'B14',
        'AI5+':'A13',
        'AI5-':'C13',
        'AI6+':'A12',
        'AI6-':'B12',
        'AI7+':'A11',
        'AI7-':'B11',
        }

# For some reason shell=True must be used for Windows but not linux (subprocess.check_output)
if os.name=='posix':
	shBool=False
else:
	shBool=True

def co(cmd):
        """ Execute command line instructions. Used here to call Xilink compiling tools. """
        return subprocess.check_output(cmd,shell=shBool)

def veri2bit(projname,dirpath):
        """ Compile verilog files to create a .xsvf configuration file.
                Inputs should be the name of the project (e.g. sync) and a path/to/a/directory containing
                the verilog, projname.ucf, projname.prj and projname.xst files.
                Output is a list containing the logs."""

        # Clean and create a temp directory to store compilation files.
        tmpdir=dirpath+'/veri2bit_files'
        if os.path.exists(tmpdir):
                shutil.rmtree(tmpdir)
        os.mkdir(tmpdir)

        # Moves all necessary files in the directory
        for file in glob.glob(dirpath+'/*.v'):
                shutil.copy(file,tmpdir)
        shutil.copy(dirpath+'/'+projname+'.ucf',tmpdir)
        shutil.copy(dirpath+'/'+projname+'.xst',tmpdir)
        shutil.copy(dirpath+'/'+projname+'.prj',tmpdir)
        shutil.copy(dirpath+'/'+'impact.batch',tmpdir)

        # Save current directory and move to temp directory.
        olddir=os.getcwd()
        os.chdir(tmpdir)

        # Call Xilinx tool (which have to be properly installed* and licensed).
        # *specific environment variables have to be set
        with open(projname+'.xst', 'a') as xstfile:
            xstfile.write('-p '+chipname)
        xst_output = co(['xst','-ifn',projname+'.xst'])
        print(xst_output)

        ngdbuild_output = co(['ngdbuild','-p',chipname,'-uc',projname+'.ucf',projname+'.ngc'])
        print(ngdbuild_output)

        map_output = co('map -detail -pr b'.split() + [projname+'.ngd'])
        print(map_output)

        par_output = co('par -ol high -w'.split()+ [projname+'.ncd',projname+'.par.ncd'])
        print(par_output)

        bitgen_output = co('bitgen -w -g StartupClk:Cclk'.split()+ [projname+'.par.ncd',projname+'.bit'])
        print(bitgen_output)

        # Generate .xsvf file. (used by FPGALink loading utilities).      
        impact_output = co(['impact','-batch','impact.batch'])
        print(impact_output)     

        # Save projname.bit and projname.xsvf file and go back to old working directory.
        shutil.copy(projname+'.bit','..')
        shutil.copy(projname+'.xsvf','..')
        os.chdir(olddir)

        return [xst_output,ngdbuild_output,map_output,par_output,bitgen_output,impact_output]

def bit2Fpga(path,dev='sync'):
        if dev=='sync':
            vp=syncVp
        elif dev=='APDacq':
            vp=APDacqVp
        else:
            vp=dev
            
        try:
            handle = fpgalink2.flOpen(vp)
            fpgalink2.flPlayXSVF(handle, path)
            fpgalink2.flClose(handle)
            print("Successfully loaded %s on device %s."%(path,vp))
        except fpgalink2.FLException, ex:
            print ex
        

def loadFirmware(vp=syncVp,ivp=originalVp):
        ''' Loads the firmware provided by FpgaLink and optionally redefines the board IDs.
        Has to be called each time the board is reset.
        vp are the vendor and product IDs used.
        ivp are the initial ids, if no board is found with vp, firmware will be loaded to ivp and the IDs changed to vp.
        '''
        print "Attempting to open connection to FPGALink device %s..." % vp
        try:
            handle = fpgalink2.flOpen(vp)
            fpgalink2.flClose(handle)
        except fpgalink2.FLException, ex:
            if ( ivp ):
                jtagPort = "D0234"
                print "Connection to %s failed, loading firmware into %s..." % (vp,ivp)
                fpgalink2.flLoadStandardFirmware(ivp, vp, jtagPort);

                print "Awaiting renumeration..."
                if ( not fpgalink2.flAwaitDevice(vp, 600) ):
                    raise fpgalink2.FLException("FPGALink device did not renumerate properly as %s" % vp)

                print "Attempting to open connection to FPGALink device %s again..." % vp
                handle = fpgalink2.flOpen(vp)
                fpgalink2.flClose(handle)
            else:
                raise fpgalink2.FLException("Could not open FPGALink device at %s and no initial VID:PID was supplied" % vp)


### Below: because two boards with identical <VID:PID> are used in our lab. These IDs are redefined to two different values.
def redef_syncID():
        ''' Redefine <VID:PID> for first (synchronizer) FPGA board, to distinguish from the other one.
        Only one board with original 1443:0007 id should be plugged when running this. '''
        loadFirmware(vp=syncVp,ivp=originalVp)

def redef_acqID():
        ''' Redefine <VID:PID> for second (APD acquisition) FPGA board, to distinguish from the other one.
        Only one board with original 1443:0007 id should be plugged when running this. '''
        loadFirmware(vp=APDacqVp,ivp=originalVp)

def redefine_IDs():
        raw_input('Unplug Synchronizer FPGA board then press enter. (Make sure APDacquisition board is plugged.')
        redef_acqID()
        raw_input('Now plug the synchronizer FPGA.')
        redef_syncID()
     
if __name__ =="__main__":
        print('Running main.py')
        #loadFirmware()
        #bit2Fpga('simplesync_files/sync.xsvf')
        #veri2bit('sync','simplesync_files')
        
        #veri2bit('APDacq','APDacq_files')
        #bit2Fpga('APDacq_files/APDacq.xsvf',dev='APDacq')
        
        #redefine_IDs()
        #redef_acqID()
        #redef_syncID()
