#!/usr/bin/env python
"""
Tools the write the files needed to compile a FPGA configuration for a synchronizer.
    Here a sequence is repeated with some parameters tunable via USB communication and a subsequence repeated during each sequence.
And tools to communicate with the FPGA once the compiled file is loaded.

Defines:
    write_verilog, taking a timing sequence+subsequence for all outputs as input and writing a coresponding verilog file as output.
    wrtie_ucf, writing the usf contraint file, mostly assigning all outputs of the FPGA design to the proper BNC plug names

    regWrite, to write some value in memories of the FPGA, and different tools using regWrite to use specific registers.    

Times were initially in the:
    (sec,ms,musec,nanosec) format.
    this is not followed anymore, now (sec,ms,musec,10nanosec) for fixed 100Mhz frequency, other frequency easily used with (sec,ms,musec,nanosec) (see below)
    'n' and 'ti' or other strings can be used as input

Uses files in sync_files/, (e.g. verilog code from intro.v_part, mid.v_part and end.v_part)
writes in sync.v
"""

"""
Note on frequency management. Depending on frequency, exact output or some approximation. Input times expressed in '10ns' or 'ns' unit.
convert_time(...) function converts the input time format to propoer unit.
For different frequencies than 100MHz, DCM or PLL has to be set up.
Different approaches:
    1.frequency=frequency={'style':'fixed 100Mhz','base_time_unit':'10ns'}
    
        Frequency is fixed to 100MHz,
        Input is in 10ns unit (string or number).
        verilog code style:
            if counter == string

    2.frequency={'style'='rational','base_time_unit'='ns','d'=f_Hz/10**9}
        Frequency has a practical value (integer value for d, see the following).
        Input expressed in ns.
        verilog code style:
            if counter == string /d
                f:100Mhz d=10
                f:200Mhz d=5
                f:500Mhz d=2
                f:10MHz d=100
        Note so sure this division by a constant integer is always of for synthesis.

    3.frequency={'style'='float','base_time_unit'='ns','f_GHz'= some_float_value, 'exponent'=some_int_value}
        Any frequency, but approximation.
        input expressed in ns.
        f_GHz: frequency expressed in GHz
        exponent: also noted z
        f_2_z=int(f_GHz*2**z)

        input_time*f_2_z = nuumber_of_cycles*2**z

        verilog code style:
            if counter == (string * f_2_z)>>z    (>>z shift operator, z bits <-> /2**z)

        (relative error if f=100Mhz, z=10 (2**10=1024) -> 0.4%, z=16 -> 9e-5, z=32 -> 1e-9)
"""
numlist=['0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F',
       'G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V']

import main
from main import fpgalink2 as fpgalink2

fpgaboard_id=main.syncVp# was redefined from original '1443:0007'

# Fpga frequency.
frequency={'style':'fixed 100Mhz','base_time_unit':'10ns'}
#frequency={'style':'rational','base_time_unit':'ns','d':10}
#frequency={'style':'float','base_time_unit':'ns','f_GHz':0.1, 'exponent':16}

######## Path management for files to read/write
# if the module is not imported __file__ is not defined, or dirname(__file__)='', otherwise dirname(__file__)='/path/to/dir'
import os.path
if '__file__' in locals():
    base=os.path.dirname(__file__)+'/'
else:
    base=''
base += 'syncComSubseq_files'

def sync_files():
    return base

########## Compile tools
def prepare(period,outputs_sequences,outputs_bnc_names,outputs_subsequences):
    ''' Write the necessary files and compile. '''
    write_verilog(period,outputs_sequences,outputs_subsequences)
    write_ucf(outputs_bnc_names)
    main.veri2bit('sync',base)
    
def convert_time(input_format,base_time_unit):
    ''' Converts a time in user input format to a string expressing the time in '10ns' or 'ns' unit (the string may contain operations such as addition/substraction)
        base_time_unit is either '10ns' or 'ns'
        function used by write_verilog through counter_eq_string
    '''
    # Here the user input format is (a,b,c,d) where a is seconds, b milliseconds, c microseconds and d either nanosecs or 10ns dependending on frequency['base_time_unit']. a,b,c,d are a number or string.

    #Different behaviour if input is a string or number
    num=0
    string=''

    # Define the multiplier for each part of input (e.g. for ms part and 10ns base unit, multiplier is 10**5)    
    if base_time_unit=='ns':
        mul=[10**9,10**6,10**3,1]
    elif base_time_unit=='10ns':
        mul=[10**8,10**5,10**2,1]
    
    for j,time in enumerate(input_format):
        if isinstance(time,(int,long)):     #if input is a number
            num+=time*mul[j]
        else:                               #input is a string
            string+='+('+str(time)+')*'+str(mul[j])
            
    s=str(int(num))+string

    for i in range(32):
        num=numlist[i]
        s_old='t%s'%num
        s_new='(t%s_reg[15] ? -t%s : t%s)'%(num,num,num)
        
        s=s.replace(s_old,s_new)
    
    return s

def counter_eq_string(input_format,frequency):
    ''' Depending of frequency['style'] and given the time in input_format returns the proper string the counter should be compared to. '''
    time_string=convert_time(input_format,frequency['base_time_unit'])
                                                    
    if frequency['style']=='fixed 100Mhz':
        output_string=time_string
    if frequency['style']=='rational':
        output_string='('+time_string+'/%s)'% frequency['d']
    if frequency['style']=='float':
        f_GHz=frequency['f_GHz']
        z=frequency['exponent']
        f_2_z=int(f_GHz*2**z)

        output_string='(('+time_string+'*'+str(f_2_z)+')>>'+str(z)+')'

    return output_string

def ti_string1():
    string=''
    s="""
            reg[15:0]   t%s_reg             = 16'h0000;
            wire[15:0]  t%s_next;
            wire[14:0]  t%s;        //ti absolute value
            //wire[14:0]  t%s_s;      //ti sign -> all 0 positive, all 1, negative
            assign      t%s=t%s_reg[14:0];
            //assign      t%s_s={15{t%s_reg[15]}};
    """
    for i in range(32):
        num=numlist[i]
        string+=s%(num,num,num,num,num,num,num,num)

    return string

def ti_string2():
    string=''
    s="""
			t%s_reg <= t%s_next;    """
    for i in range(32):
        num=numlist[i]
        string+=s%(num,num)

    return string

def ti_string3():
    string=''
    s="""
		assign t%s_next[7:0] =
			(chanAddr == 7'b1%s0 && h2fValid == 1'b1) ? h2fData : t%s_reg[7:0];
		assign t%s_next[15:8] =
			(chanAddr == 7'b1%s1 && h2fValid == 1'b1) ? h2fData : t%s_reg[15:8];      
    """
    for i in range(32):
        num=numlist[i]
        b=bin(i)[2:]
        ss='0'*(5-len(b))+b
        string+=s%(num,ss,num,num,ss,num)

    return string

def write_verilog(period,outputs_sequences,outputs_subsequences):
    outString=""

    s=counter_eq_string(period,frequency)                   
    outString += """
            else if(counter == %s) // if t as reached period, reset outputs and counter
                begin
                    counter=0;
                    out=0;
                    n=n + 1;
                    trig_is_low=0;
                    running=0;
                    sub_running=0;
                end
            else
                begin"""% s
                
    for i, output in enumerate(outputs_sequences):
        else_string=""
        outString+="""
            // output %d"""% i
        if i<32:            # if i<16 manualEnable allows to manually control output i.
            outString+="""
                    if(manualEnable[%d]==1)
                        begin
                            out[%d]=manualValues[%d];
                        end
                    """% (i,i,i)
            else_string+="else "
        for date in output:
            s=counter_eq_string(date,frequency)
            outString += """
                    %sif(counter == %s)           
                        begin
                           out[%d]=out[%d]+1;
                        end
                        """% (else_string, s, i, i)

    subseqString=""
    subseq_endstring=""
    for i, output in enumerate(outputs_subsequences):
        if(len(output)!=0):
            subseqString+="""
                    // output %d, subsequence"""%i

            for index,date in enumerate(output):
                subseqString+="\n\t\t\t\t\t\tif("
                if i<32:
                    subseqString+="(manualEnable[%d]==0) & "% i
                s=counter_eq_string(date,frequency)
                newvalue=((index+1)%2)
                #print(s)
                subseqString += """((sub_counter == %s)|(sub_counter == %s)|(sub_counter == %s)) )          
                            begin
                                out[%d]=%d;
                            end
                            """% (s, '('+s+'+Tsub)', '('+s+'-Tsub)', i, newvalue)
            subseq_endstring += "\t\t\t\t\t\t\t\t\tout[%d]=0; //Outputs unused in subseqs must be set to zero after end of subseq.\n"%i

    intro = open(base+"/intro.v_part","r")
    intro_format = intro.read() % (ti_string1(),ti_string2(),ti_string3())
    mid = open(base+"/mid.v_part","r")
    mid_format=mid.read() % counter_eq_string((0,'t0','tsub_musec',0),frequency)
    end = open(base+"/end.v_part","r")
    end_format=end.read() % subseq_endstring
    verilogString = intro_format + outString + mid_format + subseqString + end_format

    intro.close()
    mid.close()
    end.close()

    verilogfile = open(base+"/sync.v", "w")
    verilogfile.write(verilogString)
    verilogfile.close()
    
def write_ucf(ucfcfg):
    ''' Adapts the .ucf constraint file to the list ucfcfg of bnc outputs. '''
    outString=""
    for i,bnc in enumerate(ucfcfg):
        outString+='NET "outputs<%d>"         LOC = "%s" | IOSTANDARD = "LVCMOS33";\n'%(i,main.bnc2ucf[bnc])

    intro = open(base+"/intro.ucf_part","r")
    end = open(base+"/end.ucf_part","r")
    verilogString = intro.read() + outString + end.read()

    intro.close()
    end.close()

    ucffile = open(base+"/sync.ucf", "w")
    ucffile.write(verilogString)
    ucffile.close()

######### Communication tools
def getFpgaHandle():
    return fpgalink2.flOpen(fpgaboard_id)

def closeFpgaHandle(fpga_handle):
    fpgalink2.flClose(fpga_handle)

def regWrite(address,value,fpga_handle=None):
    if fpga_handle==None:
        fpga_handle=fpgalink2.flOpen(fpgaboard_id)
        fpgalink2.flWriteChannel(fpga_handle,100,address,value)
        fpgalink2.flClose(fpga_handle)
    else:
        fpgalink2.flWriteChannel(fpga_handle,100,address,value)

def regWrite32(address,outs,fpga_handle=None):
    """ Writes a list of up to 32 bits in a 4*8 bit registers, with first register at given address and the others following. """
    if fpga_handle==None:
        fpga_handle=fpgalink2.flOpen(fpgaboard_id)
        to_be_closed=True
    else:
        to_be_closed=False
    for j in range(4):
        string=''
        for i in outs[8*j:8*(j+1)]:
            string=str(i)+string
        if string!='':
            value=int(string,2)
        else:
            value=0
        regWrite(address+j,value,fpga_handle)
    if to_be_closed:
        fpgalink2.flClose(fpga_handle)
                
def start(fpga_handle=None):
    """Starts the Fpga sequence, using FpgaLink and convention that reg0[0] enables the sequence to run."""
    regWrite(0,1,fpga_handle)

def stop(fpga_handle=None):
    """Stops the Fpga sequence, using FpgaLink and convention that reg0[0] enables the sequence to run."""
    regWrite(0,0,fpga_handle)
    
def autotrig(allow,fpga_handle=None):
    """ Turn on/off continous trig. """
    if allow:
        regWrite(1,1,fpga_handle)
    else:
        regWrite(1,0,fpga_handle)

def manualEnable(outs,fpga_handle=None):
    """Takes a list of 1 to 32 bits allowing manual control of the first outputs instead of the preprogrammed sequence."""
    regWrite32(8,outs,fpga_handle)
    
def manualValues(outs,fpga_handle=None):
    """Takes a list of 1 to 32 bits allowing manual control of the first outputs instead of the preprogrammed sequence."""
    regWrite32(12,outs,fpga_handle)

def redefine_tparams(address,value,fpga_handle=None):
    if fpga_handle==None:
        fpga_handle=fpgalink2.flOpen(fpgaboard_id)
        to_be_closed=True
    else:
        to_be_closed=False

    if abs(value)>=2**15:
        print ("Warning: abs(t) value is higher than 2**15")

    if value >=0: #write in ti_p, set ti_n to zero
        regWrite(2**6+address*2,value%(2**8),fpga_handle)
        regWrite(2**6+address*2+1,value//(2**8),fpga_handle)
    else:
        regWrite(2**6+address*2,abs(value)%(2**8),fpga_handle)
        regWrite(2**6+address*2+1,(abs(value)+2**15)//(2**8),fpga_handle)

    if to_be_closed:
        fpgalink2.flClose(fpga_handle)

def redefine_tsub(value,fpga_handle=None):
    if fpga_handle==None:
        fpga_handle=fpgalink2.flOpen(fpgaboard_id)
        to_be_closed=True
    else:
        to_be_closed=False
        
    regWrite(2**5,value%(2**8),fpga_handle)
    regWrite(2**5+1,value//(2**8),fpga_handle)

def redefine_nsubmax(value,fpga_handle=None):
    if fpga_handle==None:
        fpga_handle=fpgalink2.flOpen(fpgaboard_id)
        to_be_closed=True
    else:
        to_be_closed=False
        
    regWrite(2**5+2,value%(2**8),fpga_handle)
    regWrite(2**5+3,value//(2**8),fpga_handle)

    if to_be_closed:
        fpgalink2.flClose(fpga_handle)

def redefine_Tsub(value,fpga_handle=None):
    if fpga_handle==None:
        fpga_handle=fpgalink2.flOpen(fpgaboard_id)
        to_be_closed=True
    else:
        to_be_closed=False
        
    regWrite(2**5+4,value%(2**8),fpga_handle)
    regWrite(2**5+5,value//(2**8),fpga_handle)

    if to_be_closed:
        fpgalink2.flClose(fpga_handle)

def redefine_subsequence(tsub,nsubmax,Tsub,fpga_handle=None):
    if fpga_handle==None:
        fpga_handle=fpgalink2.flOpen(fpgaboard_id)
        to_be_closed=True
    else:
        to_be_closed=False
        
    redefine_tsub(tsub,fpga_handle)
    redefine_nsubmax(nsubmax,fpga_handle)
    redefine_Tsub(Tsub,fpga_handle)

    if to_be_closed:
        fpgalink2.flClose(fpga_handle)

if __name__ =="__main__":
    period = (10,0,0,0)
    outputs = [# (secs, millisecs, microsecs, 10nanosecs)
    # output 0,
    [],
    # output 1
    [(1,0,0,0),(1,'(1+2*n)*200',0,0)],
    # output 2
    [(2,0,0,0),('2+t1+t2',0,0,0)],
    # output 3
    [(3,0,0,0),(4,0,0,0)],
    # output 4
    [(4,0,0,0),(5,0,0,0)],
    # output 5
    [(5,0,0,0),(6,0,0,0)],
    # output 6
    [(6,0,0,0),(7,0,0,0)],
    # output 7
    [(7,0,0,0),(8,0,0,0)],
    ]
    
    outputs_subseq = [[(0,0,1,0),(0,0,'t3',0)],[],[],[],[],[],[],[]]

    write_verilog(period,outputs,outputs_subseq)
    
    l=[u'LED0', u'LED1', u'LED2', u'LED3', u'LED4', u'LED5', u'LED6', u'LED7']
    write_ucf(l)
    
    main.veri2bit('sync',base)
    main.bit2Fpga('syncComSubseq_files/sync.xsvf')
    
    start()
    autotrig(True)
    redefine_subsequence(10,100,10000)
    redefine_tparams(1,1)
    manualEnable([0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
    manualValues([0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
    
