// 	Outputs:
//		outputs[i] used to control the experiment.
// 
//	Inputs:
//		rst stops the sequence and put every output to low,
//		clk: clock from the Fpga board,
//		trig: trig in, (fixed to AI0+=A16 for now: 2014-06-24)
//		fx2....: inputs and outputs for usb communication
//
// 	Communication with PC:
//		When writing:
//		reg0[0] controls if the sequence is running or not (idem as rst) address: 0
//		reg1[0] allows continuous trigging, address: 1
//		manualEnable[i] (i from 0 to 31) controls if output i is controlled manually(1) or by the preprogrammed sequence (0)   address : 0001000, 1001, 1010, 1011
//		manualValues[i] is the output i value when controlled manually					address : 1100, 1101, 1110, 1111
//		tsub_musec, nsubmax, Tsub: 16 bits registers for subsequences, 					address: 0100000 to 0100101
// 		
//		t1_p, t1_n to tf_p tf_n are registers defining different times in the sequence, values defined with communication with the PC (use t1_p for positive value, t1_n for negative)
//		address: 1000000, 1000001 for t0_p, 1100000, 1100001 for t0_n ...
//
//		All regs are initially at 0
//
//		When reading:
//		reg1 has the same meaning than above.
//		reg0 returns the current outputs value (for outputs 0 to 7).
//
//		For now (2014-06-24) the frequency is fixed to 100Mhz
//

module sync_main(
	// In those parentheses, all inputs/outputs of the main module are listed. After the parentheses is what happens inside the module.

	// main inputs and outputs
    output wire [31:0] outputs,
    input wire clk,
    input wire rst,	// this resets output pins to 0
	input wire trig,

	// inputs and outputs for USB communication	
	// FX2 (USB) interface -----------------------------------------------------------------------------
		input  wire      fx2Clk_in,     // 48MHz clock from FX2
		output wire[1:0] fx2Addr_out,   // select FIFO: "10" for EP6OUT, "11" for EP8IN
		inout  wire[7:0] fx2Data_io,    // 8-bit data to/from FX2

		// When EP6OUT selected: -> PC to FPGA
		output wire      fx2Read_out,   // asserted (active-low) when reading from FX2
		output wire      fx2OE_out,     // asserted (active-low) to tell FX2 to drive bus
		input  wire      fx2GotData_in, // asserted (active-high) when FX2 has data for us

		// When EP8IN selected: -> FPGA to PC
		output wire      fx2Write_out,  // asserted (active-low) when writing to FX2
		input  wire      fx2GotRoom_in, // asserted (active-high) when FX2 has room for more data from us
		output wire      fx2PktEnd_out // asserted (active-low) when a host read needs to be committed early
	
    );

	// General usage registers ///////////////
    reg [31:0] counter;		// 32bits counter defining the time of an experiment sequence, at 100Mhz: 43s in total
    reg [31:0] n; 			// 32 bit cycle counter, counts the sequences that passed, can be used to vary some parameter depending on n, up to 10e6 cycles
    reg [31:0] out;			// outputs
	assign outputs[31:0] = out[31:0];
	
	// For trig in
	reg running;			// for triggable sequence, is the sequence running?
	reg trig_is_low;		// for triggable sequence, used for trig edge detection

	// Registers used for subsequences : at time tsub of a sequence, a series of nsubmax subsequences of duration Tsub starts.
	// internal regs
	reg sub_running;		// boolean, are the subsequences running
	reg [31:0] nsub;		// counts the number of subsequences
	reg [31:0] sub_counter; // internal counter of a subsequence
		//user parameters
	reg [15:0] tsub_musec= 16'h0000;	// time of the main sequence when subsequences start (sub_running becomes true) - in musec, (max 65536 values)
	reg [15:0] nsubmax= 16'h0000; // maximal value for nsub (when reached sub_running becomes false)
	reg [15:0] Tsub= 16'h0000; 	// maximal value for sub_counter (=duration of a subsequence), when reached sub_counter goes back to zero and nsub is incremented
	
	///////////////////////////
	//// USB communication ////
	///////////////////////////
		// Registers accessible for usb communication, listed here if not already defined. each register reg should have a coresponding wire reg_next 

			// main registers
			reg[7:0]   reg0              = 8'h00;
			wire[7:0]  reg0_next;
			reg[7:0]   reg1              = 8'h00;
			wire[7:0]  reg1_next;
			reg[31:0]   manualEnable	= 32'h00000000;
			wire[31:0]  manualEnable_next;
			reg[31:0]   manualValues	= 32'h00000000;
			wire[31:0]  manualValues_next;
 	 
			//registers used for post-compile&load timing definitions
			
            reg[15:0]   t0_reg             = 16'h0000;
            wire[15:0]  t0_next;
            wire[14:0]  t0;        //ti absolute value
            //wire[14:0]  t0_s;      //ti sign -> all 0 positive, all 1, negative
            assign      t0=t0_reg[14:0];
            //assign      t0_s={15{t0_reg[15]}};
    
            reg[15:0]   t1_reg             = 16'h0000;
            wire[15:0]  t1_next;
            wire[14:0]  t1;        //ti absolute value
            //wire[14:0]  t1_s;      //ti sign -> all 0 positive, all 1, negative
            assign      t1=t1_reg[14:0];
            //assign      t1_s={15{t1_reg[15]}};
    
            reg[15:0]   t2_reg             = 16'h0000;
            wire[15:0]  t2_next;
            wire[14:0]  t2;        //ti absolute value
            //wire[14:0]  t2_s;      //ti sign -> all 0 positive, all 1, negative
            assign      t2=t2_reg[14:0];
            //assign      t2_s={15{t2_reg[15]}};
    
            reg[15:0]   t3_reg             = 16'h0000;
            wire[15:0]  t3_next;
            wire[14:0]  t3;        //ti absolute value
            //wire[14:0]  t3_s;      //ti sign -> all 0 positive, all 1, negative
            assign      t3=t3_reg[14:0];
            //assign      t3_s={15{t3_reg[15]}};
    
            reg[15:0]   t4_reg             = 16'h0000;
            wire[15:0]  t4_next;
            wire[14:0]  t4;        //ti absolute value
            //wire[14:0]  t4_s;      //ti sign -> all 0 positive, all 1, negative
            assign      t4=t4_reg[14:0];
            //assign      t4_s={15{t4_reg[15]}};
    
            reg[15:0]   t5_reg             = 16'h0000;
            wire[15:0]  t5_next;
            wire[14:0]  t5;        //ti absolute value
            //wire[14:0]  t5_s;      //ti sign -> all 0 positive, all 1, negative
            assign      t5=t5_reg[14:0];
            //assign      t5_s={15{t5_reg[15]}};
    
            reg[15:0]   t6_reg             = 16'h0000;
            wire[15:0]  t6_next;
            wire[14:0]  t6;        //ti absolute value
            //wire[14:0]  t6_s;      //ti sign -> all 0 positive, all 1, negative
            assign      t6=t6_reg[14:0];
            //assign      t6_s={15{t6_reg[15]}};
    
            reg[15:0]   t7_reg             = 16'h0000;
            wire[15:0]  t7_next;
            wire[14:0]  t7;        //ti absolute value
            //wire[14:0]  t7_s;      //ti sign -> all 0 positive, all 1, negative
            assign      t7=t7_reg[14:0];
            //assign      t7_s={15{t7_reg[15]}};
    
            reg[15:0]   t8_reg             = 16'h0000;
            wire[15:0]  t8_next;
            wire[14:0]  t8;        //ti absolute value
            //wire[14:0]  t8_s;      //ti sign -> all 0 positive, all 1, negative
            assign      t8=t8_reg[14:0];
            //assign      t8_s={15{t8_reg[15]}};
    
            reg[15:0]   t9_reg             = 16'h0000;
            wire[15:0]  t9_next;
            wire[14:0]  t9;        //ti absolute value
            //wire[14:0]  t9_s;      //ti sign -> all 0 positive, all 1, negative
            assign      t9=t9_reg[14:0];
            //assign      t9_s={15{t9_reg[15]}};
    
            reg[15:0]   tA_reg             = 16'h0000;
            wire[15:0]  tA_next;
            wire[14:0]  tA;        //ti absolute value
            //wire[14:0]  tA_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tA=tA_reg[14:0];
            //assign      tA_s={15{tA_reg[15]}};
    
            reg[15:0]   tB_reg             = 16'h0000;
            wire[15:0]  tB_next;
            wire[14:0]  tB;        //ti absolute value
            //wire[14:0]  tB_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tB=tB_reg[14:0];
            //assign      tB_s={15{tB_reg[15]}};
    
            reg[15:0]   tC_reg             = 16'h0000;
            wire[15:0]  tC_next;
            wire[14:0]  tC;        //ti absolute value
            //wire[14:0]  tC_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tC=tC_reg[14:0];
            //assign      tC_s={15{tC_reg[15]}};
    
            reg[15:0]   tD_reg             = 16'h0000;
            wire[15:0]  tD_next;
            wire[14:0]  tD;        //ti absolute value
            //wire[14:0]  tD_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tD=tD_reg[14:0];
            //assign      tD_s={15{tD_reg[15]}};
    
            reg[15:0]   tE_reg             = 16'h0000;
            wire[15:0]  tE_next;
            wire[14:0]  tE;        //ti absolute value
            //wire[14:0]  tE_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tE=tE_reg[14:0];
            //assign      tE_s={15{tE_reg[15]}};
    
            reg[15:0]   tF_reg             = 16'h0000;
            wire[15:0]  tF_next;
            wire[14:0]  tF;        //ti absolute value
            //wire[14:0]  tF_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tF=tF_reg[14:0];
            //assign      tF_s={15{tF_reg[15]}};
    
            reg[15:0]   tG_reg             = 16'h0000;
            wire[15:0]  tG_next;
            wire[14:0]  tG;        //ti absolute value
            //wire[14:0]  tG_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tG=tG_reg[14:0];
            //assign      tG_s={15{tG_reg[15]}};
    
            reg[15:0]   tH_reg             = 16'h0000;
            wire[15:0]  tH_next;
            wire[14:0]  tH;        //ti absolute value
            //wire[14:0]  tH_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tH=tH_reg[14:0];
            //assign      tH_s={15{tH_reg[15]}};
    
            reg[15:0]   tI_reg             = 16'h0000;
            wire[15:0]  tI_next;
            wire[14:0]  tI;        //ti absolute value
            //wire[14:0]  tI_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tI=tI_reg[14:0];
            //assign      tI_s={15{tI_reg[15]}};
    
            reg[15:0]   tJ_reg             = 16'h0000;
            wire[15:0]  tJ_next;
            wire[14:0]  tJ;        //ti absolute value
            //wire[14:0]  tJ_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tJ=tJ_reg[14:0];
            //assign      tJ_s={15{tJ_reg[15]}};
    
            reg[15:0]   tK_reg             = 16'h0000;
            wire[15:0]  tK_next;
            wire[14:0]  tK;        //ti absolute value
            //wire[14:0]  tK_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tK=tK_reg[14:0];
            //assign      tK_s={15{tK_reg[15]}};
    
            reg[15:0]   tL_reg             = 16'h0000;
            wire[15:0]  tL_next;
            wire[14:0]  tL;        //ti absolute value
            //wire[14:0]  tL_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tL=tL_reg[14:0];
            //assign      tL_s={15{tL_reg[15]}};
    
            reg[15:0]   tM_reg             = 16'h0000;
            wire[15:0]  tM_next;
            wire[14:0]  tM;        //ti absolute value
            //wire[14:0]  tM_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tM=tM_reg[14:0];
            //assign      tM_s={15{tM_reg[15]}};
    
            reg[15:0]   tN_reg             = 16'h0000;
            wire[15:0]  tN_next;
            wire[14:0]  tN;        //ti absolute value
            //wire[14:0]  tN_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tN=tN_reg[14:0];
            //assign      tN_s={15{tN_reg[15]}};
    
            reg[15:0]   tO_reg             = 16'h0000;
            wire[15:0]  tO_next;
            wire[14:0]  tO;        //ti absolute value
            //wire[14:0]  tO_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tO=tO_reg[14:0];
            //assign      tO_s={15{tO_reg[15]}};
    
            reg[15:0]   tP_reg             = 16'h0000;
            wire[15:0]  tP_next;
            wire[14:0]  tP;        //ti absolute value
            //wire[14:0]  tP_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tP=tP_reg[14:0];
            //assign      tP_s={15{tP_reg[15]}};
    
            reg[15:0]   tQ_reg             = 16'h0000;
            wire[15:0]  tQ_next;
            wire[14:0]  tQ;        //ti absolute value
            //wire[14:0]  tQ_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tQ=tQ_reg[14:0];
            //assign      tQ_s={15{tQ_reg[15]}};
    
            reg[15:0]   tR_reg             = 16'h0000;
            wire[15:0]  tR_next;
            wire[14:0]  tR;        //ti absolute value
            //wire[14:0]  tR_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tR=tR_reg[14:0];
            //assign      tR_s={15{tR_reg[15]}};
    
            reg[15:0]   tS_reg             = 16'h0000;
            wire[15:0]  tS_next;
            wire[14:0]  tS;        //ti absolute value
            //wire[14:0]  tS_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tS=tS_reg[14:0];
            //assign      tS_s={15{tS_reg[15]}};
    
            reg[15:0]   tT_reg             = 16'h0000;
            wire[15:0]  tT_next;
            wire[14:0]  tT;        //ti absolute value
            //wire[14:0]  tT_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tT=tT_reg[14:0];
            //assign      tT_s={15{tT_reg[15]}};
    
            reg[15:0]   tU_reg             = 16'h0000;
            wire[15:0]  tU_next;
            wire[14:0]  tU;        //ti absolute value
            //wire[14:0]  tU_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tU=tU_reg[14:0];
            //assign      tU_s={15{tU_reg[15]}};
    
            reg[15:0]   tV_reg             = 16'h0000;
            wire[15:0]  tV_next;
            wire[14:0]  tV;        //ti absolute value
            //wire[14:0]  tV_s;      //ti sign -> all 0 positive, all 1, negative
            assign      tV=tV_reg[14:0];
            //assign      tV_s={15{tV_reg[15]}};
    
			/*
			Up here registers ti_p and ti_n.
			*/
			
			// subsequence parameters
			wire[15:0] tsub_next;
			wire[15:0] nsubmax_next;
			wire[15:0] Tsub_next;
	
			// Channel read/write interface -----------------------------------------------------------------
			wire[6:0]  chanAddr;  // the selected channel (0-127)

			// Host >> FPGA pipe:
			wire[7:0]  h2fData;   // data lines used when the host writes to a channel
			wire       h2fValid;  // '1' means "on the next clock rising edge, please accept the data on h2fData_out"
			wire       h2fReady;  // channel logic can drive this low to say "I'm not ready for more data yet"

			// Host << FPGA pipe:
			wire[7:0]  f2hData;   // data lines used when the host reads from a channel
			wire       f2hValid;  // channel logic can drive this low to say "I don't have data ready for you"
			wire       f2hReady;  // '1' means "on the next clock rising edge, put your next byte of data on f2hData_in"
			// ----------------------------------------------------------------------------------------------

			// Needed so that the comm_fpga_fx2 module can drive both fx2Read_out and fx2OE_out
			wire       fx2Read;

		// Infer registers
		always @(posedge fx2Clk_in)
		begin
			reg0 <= reg0_next;
			reg1 <= reg1_next;
			manualEnable <= manualEnable_next;
			manualValues <= manualValues_next;			
			
			t0_reg <= t0_next;    
			t1_reg <= t1_next;    
			t2_reg <= t2_next;    
			t3_reg <= t3_next;    
			t4_reg <= t4_next;    
			t5_reg <= t5_next;    
			t6_reg <= t6_next;    
			t7_reg <= t7_next;    
			t8_reg <= t8_next;    
			t9_reg <= t9_next;    
			tA_reg <= tA_next;    
			tB_reg <= tB_next;    
			tC_reg <= tC_next;    
			tD_reg <= tD_next;    
			tE_reg <= tE_next;    
			tF_reg <= tF_next;    
			tG_reg <= tG_next;    
			tH_reg <= tH_next;    
			tI_reg <= tI_next;    
			tJ_reg <= tJ_next;    
			tK_reg <= tK_next;    
			tL_reg <= tL_next;    
			tM_reg <= tM_next;    
			tN_reg <= tN_next;    
			tO_reg <= tO_next;    
			tP_reg <= tP_next;    
			tQ_reg <= tQ_next;    
			tR_reg <= tR_next;    
			tS_reg <= tS_next;    
			tT_reg <= tT_next;    
			tU_reg <= tU_next;    
			tV_reg <= tV_next;    
			//here ti_next strings
			tsub_musec <= tsub_next;
			nsubmax <= nsubmax_next;
			Tsub <= Tsub_next;
		end
		
		// WRITING
		// Drive register inputs for each channel when the host is writing
		// actual adress used to access a register for writing is defined here.
		// comm_fpga_fx2 inerface used 8bits data, so longer registers have to be splitted into a few parts here.
		assign reg0_next =
			(chanAddr == 7'b0000000 && h2fValid == 1'b1) ? h2fData : reg0;
		assign reg1_next =
			(chanAddr == 7'b0000001 && h2fValid == 1'b1) ? h2fData : reg1;
		assign manualEnable_next[7:0] =
			(chanAddr == 7'b0001000 && h2fValid == 1'b1) ? h2fData : manualEnable[7:0];
		assign manualEnable_next[15:8] =
			(chanAddr == 7'b0001001 && h2fValid == 1'b1) ? h2fData : manualEnable[15:8];
		assign manualEnable_next[23:16] =
			(chanAddr == 7'b0001010 && h2fValid == 1'b1) ? h2fData : manualEnable[23:16];
		assign manualEnable_next[31:24] =
			(chanAddr == 7'b0001011 && h2fValid == 1'b1) ? h2fData : manualEnable[31:24];
		assign manualValues_next[7:0] =
			(chanAddr == 7'b0001100 && h2fValid == 1'b1) ? h2fData : manualValues[7:0];
		assign manualValues_next[15:8] =
			(chanAddr == 7'b0001101 && h2fValid == 1'b1) ? h2fData : manualValues[15:8];
		assign manualValues_next[23:16] =
			(chanAddr == 7'b0001110 && h2fValid == 1'b1) ? h2fData : manualValues[23:16];
		assign manualValues_next[31:24] =
			(chanAddr == 7'b0001111 && h2fValid == 1'b1) ? h2fData : manualValues[31:24];

		
		assign t0_next[7:0] =
			(chanAddr == 7'b1000000 && h2fValid == 1'b1) ? h2fData : t0_reg[7:0];
		assign t0_next[15:8] =
			(chanAddr == 7'b1000001 && h2fValid == 1'b1) ? h2fData : t0_reg[15:8];      
    
		assign t1_next[7:0] =
			(chanAddr == 7'b1000010 && h2fValid == 1'b1) ? h2fData : t1_reg[7:0];
		assign t1_next[15:8] =
			(chanAddr == 7'b1000011 && h2fValid == 1'b1) ? h2fData : t1_reg[15:8];      
    
		assign t2_next[7:0] =
			(chanAddr == 7'b1000100 && h2fValid == 1'b1) ? h2fData : t2_reg[7:0];
		assign t2_next[15:8] =
			(chanAddr == 7'b1000101 && h2fValid == 1'b1) ? h2fData : t2_reg[15:8];      
    
		assign t3_next[7:0] =
			(chanAddr == 7'b1000110 && h2fValid == 1'b1) ? h2fData : t3_reg[7:0];
		assign t3_next[15:8] =
			(chanAddr == 7'b1000111 && h2fValid == 1'b1) ? h2fData : t3_reg[15:8];      
    
		assign t4_next[7:0] =
			(chanAddr == 7'b1001000 && h2fValid == 1'b1) ? h2fData : t4_reg[7:0];
		assign t4_next[15:8] =
			(chanAddr == 7'b1001001 && h2fValid == 1'b1) ? h2fData : t4_reg[15:8];      
    
		assign t5_next[7:0] =
			(chanAddr == 7'b1001010 && h2fValid == 1'b1) ? h2fData : t5_reg[7:0];
		assign t5_next[15:8] =
			(chanAddr == 7'b1001011 && h2fValid == 1'b1) ? h2fData : t5_reg[15:8];      
    
		assign t6_next[7:0] =
			(chanAddr == 7'b1001100 && h2fValid == 1'b1) ? h2fData : t6_reg[7:0];
		assign t6_next[15:8] =
			(chanAddr == 7'b1001101 && h2fValid == 1'b1) ? h2fData : t6_reg[15:8];      
    
		assign t7_next[7:0] =
			(chanAddr == 7'b1001110 && h2fValid == 1'b1) ? h2fData : t7_reg[7:0];
		assign t7_next[15:8] =
			(chanAddr == 7'b1001111 && h2fValid == 1'b1) ? h2fData : t7_reg[15:8];      
    
		assign t8_next[7:0] =
			(chanAddr == 7'b1010000 && h2fValid == 1'b1) ? h2fData : t8_reg[7:0];
		assign t8_next[15:8] =
			(chanAddr == 7'b1010001 && h2fValid == 1'b1) ? h2fData : t8_reg[15:8];      
    
		assign t9_next[7:0] =
			(chanAddr == 7'b1010010 && h2fValid == 1'b1) ? h2fData : t9_reg[7:0];
		assign t9_next[15:8] =
			(chanAddr == 7'b1010011 && h2fValid == 1'b1) ? h2fData : t9_reg[15:8];      
    
		assign tA_next[7:0] =
			(chanAddr == 7'b1010100 && h2fValid == 1'b1) ? h2fData : tA_reg[7:0];
		assign tA_next[15:8] =
			(chanAddr == 7'b1010101 && h2fValid == 1'b1) ? h2fData : tA_reg[15:8];      
    
		assign tB_next[7:0] =
			(chanAddr == 7'b1010110 && h2fValid == 1'b1) ? h2fData : tB_reg[7:0];
		assign tB_next[15:8] =
			(chanAddr == 7'b1010111 && h2fValid == 1'b1) ? h2fData : tB_reg[15:8];      
    
		assign tC_next[7:0] =
			(chanAddr == 7'b1011000 && h2fValid == 1'b1) ? h2fData : tC_reg[7:0];
		assign tC_next[15:8] =
			(chanAddr == 7'b1011001 && h2fValid == 1'b1) ? h2fData : tC_reg[15:8];      
    
		assign tD_next[7:0] =
			(chanAddr == 7'b1011010 && h2fValid == 1'b1) ? h2fData : tD_reg[7:0];
		assign tD_next[15:8] =
			(chanAddr == 7'b1011011 && h2fValid == 1'b1) ? h2fData : tD_reg[15:8];      
    
		assign tE_next[7:0] =
			(chanAddr == 7'b1011100 && h2fValid == 1'b1) ? h2fData : tE_reg[7:0];
		assign tE_next[15:8] =
			(chanAddr == 7'b1011101 && h2fValid == 1'b1) ? h2fData : tE_reg[15:8];      
    
		assign tF_next[7:0] =
			(chanAddr == 7'b1011110 && h2fValid == 1'b1) ? h2fData : tF_reg[7:0];
		assign tF_next[15:8] =
			(chanAddr == 7'b1011111 && h2fValid == 1'b1) ? h2fData : tF_reg[15:8];      
    
		assign tG_next[7:0] =
			(chanAddr == 7'b1100000 && h2fValid == 1'b1) ? h2fData : tG_reg[7:0];
		assign tG_next[15:8] =
			(chanAddr == 7'b1100001 && h2fValid == 1'b1) ? h2fData : tG_reg[15:8];      
    
		assign tH_next[7:0] =
			(chanAddr == 7'b1100010 && h2fValid == 1'b1) ? h2fData : tH_reg[7:0];
		assign tH_next[15:8] =
			(chanAddr == 7'b1100011 && h2fValid == 1'b1) ? h2fData : tH_reg[15:8];      
    
		assign tI_next[7:0] =
			(chanAddr == 7'b1100100 && h2fValid == 1'b1) ? h2fData : tI_reg[7:0];
		assign tI_next[15:8] =
			(chanAddr == 7'b1100101 && h2fValid == 1'b1) ? h2fData : tI_reg[15:8];      
    
		assign tJ_next[7:0] =
			(chanAddr == 7'b1100110 && h2fValid == 1'b1) ? h2fData : tJ_reg[7:0];
		assign tJ_next[15:8] =
			(chanAddr == 7'b1100111 && h2fValid == 1'b1) ? h2fData : tJ_reg[15:8];      
    
		assign tK_next[7:0] =
			(chanAddr == 7'b1101000 && h2fValid == 1'b1) ? h2fData : tK_reg[7:0];
		assign tK_next[15:8] =
			(chanAddr == 7'b1101001 && h2fValid == 1'b1) ? h2fData : tK_reg[15:8];      
    
		assign tL_next[7:0] =
			(chanAddr == 7'b1101010 && h2fValid == 1'b1) ? h2fData : tL_reg[7:0];
		assign tL_next[15:8] =
			(chanAddr == 7'b1101011 && h2fValid == 1'b1) ? h2fData : tL_reg[15:8];      
    
		assign tM_next[7:0] =
			(chanAddr == 7'b1101100 && h2fValid == 1'b1) ? h2fData : tM_reg[7:0];
		assign tM_next[15:8] =
			(chanAddr == 7'b1101101 && h2fValid == 1'b1) ? h2fData : tM_reg[15:8];      
    
		assign tN_next[7:0] =
			(chanAddr == 7'b1101110 && h2fValid == 1'b1) ? h2fData : tN_reg[7:0];
		assign tN_next[15:8] =
			(chanAddr == 7'b1101111 && h2fValid == 1'b1) ? h2fData : tN_reg[15:8];      
    
		assign tO_next[7:0] =
			(chanAddr == 7'b1110000 && h2fValid == 1'b1) ? h2fData : tO_reg[7:0];
		assign tO_next[15:8] =
			(chanAddr == 7'b1110001 && h2fValid == 1'b1) ? h2fData : tO_reg[15:8];      
    
		assign tP_next[7:0] =
			(chanAddr == 7'b1110010 && h2fValid == 1'b1) ? h2fData : tP_reg[7:0];
		assign tP_next[15:8] =
			(chanAddr == 7'b1110011 && h2fValid == 1'b1) ? h2fData : tP_reg[15:8];      
    
		assign tQ_next[7:0] =
			(chanAddr == 7'b1110100 && h2fValid == 1'b1) ? h2fData : tQ_reg[7:0];
		assign tQ_next[15:8] =
			(chanAddr == 7'b1110101 && h2fValid == 1'b1) ? h2fData : tQ_reg[15:8];      
    
		assign tR_next[7:0] =
			(chanAddr == 7'b1110110 && h2fValid == 1'b1) ? h2fData : tR_reg[7:0];
		assign tR_next[15:8] =
			(chanAddr == 7'b1110111 && h2fValid == 1'b1) ? h2fData : tR_reg[15:8];      
    
		assign tS_next[7:0] =
			(chanAddr == 7'b1111000 && h2fValid == 1'b1) ? h2fData : tS_reg[7:0];
		assign tS_next[15:8] =
			(chanAddr == 7'b1111001 && h2fValid == 1'b1) ? h2fData : tS_reg[15:8];      
    
		assign tT_next[7:0] =
			(chanAddr == 7'b1111010 && h2fValid == 1'b1) ? h2fData : tT_reg[7:0];
		assign tT_next[15:8] =
			(chanAddr == 7'b1111011 && h2fValid == 1'b1) ? h2fData : tT_reg[15:8];      
    
		assign tU_next[7:0] =
			(chanAddr == 7'b1111100 && h2fValid == 1'b1) ? h2fData : tU_reg[7:0];
		assign tU_next[15:8] =
			(chanAddr == 7'b1111101 && h2fValid == 1'b1) ? h2fData : tU_reg[15:8];      
    
		assign tV_next[7:0] =
			(chanAddr == 7'b1111110 && h2fValid == 1'b1) ? h2fData : tV_reg[7:0];
		assign tV_next[15:8] =
			(chanAddr == 7'b1111111 && h2fValid == 1'b1) ? h2fData : tV_reg[15:8];      
    
		//assign ti_next
		
		assign tsub_next[7:0] =
			(chanAddr == 7'b0100000 && h2fValid == 1'b1) ? h2fData : tsub_musec[7:0];			
		assign tsub_next[15:8] =
			(chanAddr == 7'b0100001 && h2fValid == 1'b1) ? h2fData : tsub_musec[15:8];
		assign nsubmax_next[7:0] =
			(chanAddr == 7'b0100010 && h2fValid == 1'b1) ? h2fData : nsubmax[7:0];			
		assign nsubmax_next[15:8] =
			(chanAddr == 7'b0100011 && h2fValid == 1'b1) ? h2fData : nsubmax[15:8];
		assign Tsub_next[7:0] =
			(chanAddr == 7'b0100100 && h2fValid == 1'b1) ? h2fData : Tsub[7:0];			
		assign Tsub_next[15:8] =
			(chanAddr == 7'b0100101 && h2fValid == 1'b1) ? h2fData : Tsub[15:8];
	
		// READING
		// Select values to return for each channel when the host is reading
		assign f2hData =
			(chanAddr == 7'b0000000) ? out[7:0] : // reading address 0 from computer outputs state of out 0 to 7
			(chanAddr == 7'b0000001) ? reg1 :
			8'h00;

		// Assert that there's always data for reading, and always room for writing
		assign f2hValid = 1'b1;
		assign h2fReady = 1'b1;                                                  //END_SNIPPET(registers)

		// CommFPGA module
		assign fx2Read_out = fx2Read;
		assign fx2OE_out = fx2Read;
		assign fx2Addr_out[1] = 1'b1;  // Use EP6OUT/EP8IN, not EP2OUT/EP4IN.
		comm_fpga_fx2 comm_fpga_fx2(
			// FX2 interface
			.fx2Clk_in(fx2Clk_in),
			.fx2FifoSel_out(fx2Addr_out[0]),
			.fx2Data_io(fx2Data_io),
			.fx2Read_out(fx2Read),
			.fx2GotData_in(fx2GotData_in),
			.fx2Write_out(fx2Write_out),
			.fx2GotRoom_in(fx2GotRoom_in),
			.fx2PktEnd_out(fx2PktEnd_out),

			// Channel read/write interface
			.chanAddr_out(chanAddr),
			.h2fData_out(h2fData),
			.h2fValid_out(h2fValid),
			.h2fReady_in(h2fReady),
			.f2hData_in(f2hData),
			.f2hValid_in(f2hValid),
			.f2hReady_out(f2hReady)
		);
	//////////////////////////////////////
	//// END - USB communication - END////
	//////////////////////////////////////

	/* //Unused for now, DCM (or PLL) can be used to generate frequencies different than 100Mhz)
	wire CLKSYS;
	wire CLK25;
	wire CLK40;
	
   dcm_all dcm(
				.CLK(clk),
				.RST(rst),
				.CLKSYS(CLKSYS),
				.CLK25(CLK25),
				.CLK40(CLK40)
	);
	*/	

	//// All the following actually describes the sequence.
    always@(posedge clk) // using main 100Mhz clock
    begin
		if(~running)
			begin
				if(trig==0) // make sure trig is low before detecting a trig posedge
					begin
						trig_is_low=1;
					end
				if((trig_is_low & trig) | reg1[0]) //reg1[0] allows continuous trigging
					begin
						running=1; //edge_detected
					end
			end				
		else
			if(rst | ~reg0[0]) //reg0[0] must be set to 1 for the sequence to run
				begin
					counter=0;
					out=0;
					n=0;
					trig_is_low=0;
					running=0;
					sub_running=0;
				end

            else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t1_reg[15] ? -t1 : t1))*100) // if t as reached period, reset outputs and counter
                begin
                    counter=0;
                    out=0;
                    n=n + 1;
                    trig_is_low=0;
                    running=0;
                    sub_running=0;
                end
            else
                begin
            // output 0
                    if(manualEnable[0]==1)
                        begin
                            out[0]=manualValues[0];
                        end
                    
                    else if(counter == 0)           
                        begin
                           out[0]=out[0]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000)           
                        begin
                           out[0]=out[0]+1;
                        end
                        
            // output 1
                    if(manualEnable[1]==1)
                        begin
                            out[1]=manualValues[1];
                        end
                    
                    else if(counter == 0)           
                        begin
                           out[1]=out[1]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t2_reg[15] ? -t2 : t2))*100)           
                        begin
                           out[1]=out[1]+1;
                        end
                        
            // output 2
                    if(manualEnable[2]==1)
                        begin
                            out[2]=manualValues[2];
                        end
                    
                    else if(counter == 0)           
                        begin
                           out[2]=out[2]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t2_reg[15] ? -t2 : t2)+(t3_reg[15] ? -t3 : t3))*100)           
                        begin
                           out[2]=out[2]+1;
                        end
                        
            // output 3
                    if(manualEnable[3]==1)
                        begin
                            out[3]=manualValues[3];
                        end
                    
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t4_reg[15] ? -t4 : t4))*100)           
                        begin
                           out[3]=out[3]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t4_reg[15] ? -t4 : t4)+(t5_reg[15] ? -t5 : t5))*100)           
                        begin
                           out[3]=out[3]+1;
                        end
                        
            // output 4
                    if(manualEnable[4]==1)
                        begin
                            out[4]=manualValues[4];
                        end
                    
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t6_reg[15] ? -t6 : t6))*100)           
                        begin
                           out[4]=out[4]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t6_reg[15] ? -t6 : t6)+(t7_reg[15] ? -t7 : t7))*100)           
                        begin
                           out[4]=out[4]+1;
                        end
                        
            // output 5
                    if(manualEnable[5]==1)
                        begin
                            out[5]=manualValues[5];
                        end
                    
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t6_reg[15] ? -t6 : t6))*100)           
                        begin
                           out[5]=out[5]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t6_reg[15] ? -t6 : t6)+10)*100)           
                        begin
                           out[5]=out[5]+1;
                        end
                        
            // output 6
                    if(manualEnable[6]==1)
                        begin
                            out[6]=manualValues[6];
                        end
                    
            // output 7
                    if(manualEnable[7]==1)
                        begin
                            out[7]=manualValues[7];
                        end
                    
            // output 8
                    if(manualEnable[8]==1)
                        begin
                            out[8]=manualValues[8];
                        end
                    
            // output 9
                    if(manualEnable[9]==1)
                        begin
                            out[9]=manualValues[9];
                        end
                    
            // output 10
                    if(manualEnable[10]==1)
                        begin
                            out[10]=manualValues[10];
                        end
                    
            // output 11
                    if(manualEnable[11]==1)
                        begin
                            out[11]=manualValues[11];
                        end
                    
                    else if(counter == 0)           
                        begin
                           out[11]=out[11]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t2_reg[15] ? -t2 : t2)+(t3_reg[15] ? -t3 : t3))*100)           
                        begin
                           out[11]=out[11]+1;
                        end
                        
            // output 12
                    if(manualEnable[12]==1)
                        begin
                            out[12]=manualValues[12];
                        end
                    
                    else if(counter == 0)           
                        begin
                           out[12]=out[12]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t2_reg[15] ? -t2 : t2)+(t3_reg[15] ? -t3 : t3))*100)           
                        begin
                           out[12]=out[12]+1;
                        end
                        
            // output 13
                    if(manualEnable[13]==1)
                        begin
                            out[13]=manualValues[13];
                        end
                    
                    else if(counter == 0)           
                        begin
                           out[13]=out[13]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t2_reg[15] ? -t2 : t2)+(t3_reg[15] ? -t3 : t3))*100)           
                        begin
                           out[13]=out[13]+1;
                        end
                        
            // output 14
                    if(manualEnable[14]==1)
                        begin
                            out[14]=manualValues[14];
                        end
                    
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t6_reg[15] ? -t6 : t6))*100)           
                        begin
                           out[14]=out[14]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t6_reg[15] ? -t6 : t6)+(t7_reg[15] ? -t7 : t7))*100)           
                        begin
                           out[14]=out[14]+1;
                        end
                        
            // output 15
                    if(manualEnable[15]==1)
                        begin
                            out[15]=manualValues[15];
                        end
                    
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t6_reg[15] ? -t6 : t6))*100)           
                        begin
                           out[15]=out[15]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t6_reg[15] ? -t6 : t6)+(t7_reg[15] ? -t7 : t7))*100)           
                        begin
                           out[15]=out[15]+1;
                        end
                        
            // output 16
                    if(manualEnable[16]==1)
                        begin
                            out[16]=manualValues[16];
                        end
                    
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t6_reg[15] ? -t6 : t6))*100)           
                        begin
                           out[16]=out[16]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t6_reg[15] ? -t6 : t6)+(t7_reg[15] ? -t7 : t7))*100)           
                        begin
                           out[16]=out[16]+1;
                        end
                        
            // output 17
                    if(manualEnable[17]==1)
                        begin
                            out[17]=manualValues[17];
                        end
                    
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t6_reg[15] ? -t6 : t6))*100)           
                        begin
                           out[17]=out[17]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t6_reg[15] ? -t6 : t6)+(t7_reg[15] ? -t7 : t7))*100)           
                        begin
                           out[17]=out[17]+1;
                        end
                        
            // output 18
                    if(manualEnable[18]==1)
                        begin
                            out[18]=manualValues[18];
                        end
                    
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+(tsub_musec)*100+((tQ_reg[15] ? -tQ : tQ))*1)           
                        begin
                           out[18]=out[18]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+(tsub_musec)*100+(nsubmax*Tsub+(tR_reg[15] ? -tR : tR))*1)           
                        begin
                           out[18]=out[18]+1;
                        end
                        
            // output 19
                    if(manualEnable[19]==1)
                        begin
                            out[19]=manualValues[19];
                        end
                    
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+(tsub_musec)*100+((tQ_reg[15] ? -tQ : tQ))*1)           
                        begin
                           out[19]=out[19]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+(tsub_musec)*100+(nsubmax*Tsub+(tR_reg[15] ? -tR : tR))*1)           
                        begin
                           out[19]=out[19]+1;
                        end
                        
            // output 20
                    if(manualEnable[20]==1)
                        begin
                            out[20]=manualValues[20];
                        end
                    
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+(tsub_musec)*100+((tQ_reg[15] ? -tQ : tQ))*1)           
                        begin
                           out[20]=out[20]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+(tsub_musec)*100+(nsubmax*Tsub+(tR_reg[15] ? -tR : tR))*1)           
                        begin
                           out[20]=out[20]+1;
                        end
                        
            // output 21
                    if(manualEnable[21]==1)
                        begin
                            out[21]=manualValues[21];
                        end
                    
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+(tsub_musec)*100+((tQ_reg[15] ? -tQ : tQ))*1)           
                        begin
                           out[21]=out[21]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+(tsub_musec)*100+(nsubmax*Tsub+(tR_reg[15] ? -tR : tR))*1)           
                        begin
                           out[21]=out[21]+1;
                        end
                        
            // output 22
                    if(manualEnable[22]==1)
                        begin
                            out[22]=manualValues[22];
                        end
                    
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+(tsub_musec)*100+((tO_reg[15] ? -tO : tO))*1)           
                        begin
                           out[22]=out[22]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+(tsub_musec+10)*100+((tO_reg[15] ? -tO : tO))*1)           
                        begin
                           out[22]=out[22]+1;
                        end
                        
            // output 23
                    if(manualEnable[23]==1)
                        begin
                            out[23]=manualValues[23];
                        end
                    
                    else if(counter == 0)           
                        begin
                           out[23]=out[23]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t1_reg[15] ? -t1 : t1)-(tH_reg[15] ? -tH : tH))*100)           
                        begin
                           out[23]=out[23]+1;
                        end
                        
            // output 24
                    if(manualEnable[24]==1)
                        begin
                            out[24]=manualValues[24];
                        end
                    
                    else if(counter == 0)           
                        begin
                           out[24]=out[24]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t1_reg[15] ? -t1 : t1)-(tH_reg[15] ? -tH : tH)+(tI_reg[15] ? -tI : tI))*100)           
                        begin
                           out[24]=out[24]+1;
                        end
                        
            // output 25
                    if(manualEnable[25]==1)
                        begin
                            out[25]=manualValues[25];
                        end
                    
                    else if(counter == 0)           
                        begin
                           out[25]=out[25]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t1_reg[15] ? -t1 : t1)-(tH_reg[15] ? -tH : tH)+(tI_reg[15] ? -tI : tI)+(tJ_reg[15] ? -tJ : tJ))*100)           
                        begin
                           out[25]=out[25]+1;
                        end
                        
            // output 26
                    if(manualEnable[26]==1)
                        begin
                            out[26]=manualValues[26];
                        end
                    
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t1_reg[15] ? -t1 : t1)-(tH_reg[15] ? -tH : tH)+(tM_reg[15] ? -tM : tM))*100)           
                        begin
                           out[26]=out[26]+1;
                        end
                        
                    else if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+((t1_reg[15] ? -t1 : t1)-(tH_reg[15] ? -tH : tH)+(tM_reg[15] ? -tM : tM)+(tN_reg[15] ? -tN : tN))*100)           
                        begin
                           out[26]=out[26]+1;
                        end
                        
            // output 27
                    if(manualEnable[27]==1)
                        begin
                            out[27]=manualValues[27];
                        end
                    
            // output 28
                    if(manualEnable[28]==1)
                        begin
                            out[28]=manualValues[28];
                        end
                    
            // output 29
                    if(manualEnable[29]==1)
                        begin
                            out[29]=manualValues[29];
                        end
                    
            // output 30
                    if(manualEnable[30]==1)
                        begin
                            out[30]=manualValues[30];
                        end
                    
				
				// subsequence
				// subsequence initialisation
				if(counter == 0+((t0_reg[15] ? -t0 : t0))*100000+(tsub_musec)*100)
					begin
						sub_running=1;
						sub_counter=0;
						nsub=0;
					end
					
				if(sub_running)
					begin
                    // output 6, subsequence
						if((manualEnable[6]==0) & ((sub_counter == 0+((tA_reg[15] ? -tA : tA)+(t8_reg[15] ? -t8 : t8))*1)|(sub_counter == (0+((tA_reg[15] ? -tA : tA)+(t8_reg[15] ? -t8 : t8))*1+Tsub))|(sub_counter == (0+((tA_reg[15] ? -tA : tA)+(t8_reg[15] ? -t8 : t8))*1-Tsub))) )          
                            begin
                                out[6]=1;
                            end
                            
						if((manualEnable[6]==0) & ((sub_counter == 0+((tB_reg[15] ? -tB : tB)+(t8_reg[15] ? -t8 : t8))*1)|(sub_counter == (0+((tB_reg[15] ? -tB : tB)+(t8_reg[15] ? -t8 : t8))*1+Tsub))|(sub_counter == (0+((tB_reg[15] ? -tB : tB)+(t8_reg[15] ? -t8 : t8))*1-Tsub))) )          
                            begin
                                out[6]=0;
                            end
                            
                    // output 7, subsequence
						if((manualEnable[7]==0) & ((sub_counter == 0+((tA_reg[15] ? -tA : tA)+(t8_reg[15] ? -t8 : t8))*1)|(sub_counter == (0+((tA_reg[15] ? -tA : tA)+(t8_reg[15] ? -t8 : t8))*1+Tsub))|(sub_counter == (0+((tA_reg[15] ? -tA : tA)+(t8_reg[15] ? -t8 : t8))*1-Tsub))) )          
                            begin
                                out[7]=1;
                            end
                            
						if((manualEnable[7]==0) & ((sub_counter == 0+((tB_reg[15] ? -tB : tB)+(t8_reg[15] ? -t8 : t8))*1)|(sub_counter == (0+((tB_reg[15] ? -tB : tB)+(t8_reg[15] ? -t8 : t8))*1+Tsub))|(sub_counter == (0+((tB_reg[15] ? -tB : tB)+(t8_reg[15] ? -t8 : t8))*1-Tsub))) )          
                            begin
                                out[7]=0;
                            end
                            
                    // output 8, subsequence
						if((manualEnable[8]==0) & ((sub_counter == 0+((tA_reg[15] ? -tA : tA)+(t8_reg[15] ? -t8 : t8))*1)|(sub_counter == (0+((tA_reg[15] ? -tA : tA)+(t8_reg[15] ? -t8 : t8))*1+Tsub))|(sub_counter == (0+((tA_reg[15] ? -tA : tA)+(t8_reg[15] ? -t8 : t8))*1-Tsub))) )          
                            begin
                                out[8]=1;
                            end
                            
						if((manualEnable[8]==0) & ((sub_counter == 0+((tB_reg[15] ? -tB : tB)+(t8_reg[15] ? -t8 : t8))*1)|(sub_counter == (0+((tB_reg[15] ? -tB : tB)+(t8_reg[15] ? -t8 : t8))*1+Tsub))|(sub_counter == (0+((tB_reg[15] ? -tB : tB)+(t8_reg[15] ? -t8 : t8))*1-Tsub))) )          
                            begin
                                out[8]=0;
                            end
                            
                    // output 9, subsequence
						if((manualEnable[9]==0) & ((sub_counter == 0+((tC_reg[15] ? -tC : tC)+(t9_reg[15] ? -t9 : t9))*1)|(sub_counter == (0+((tC_reg[15] ? -tC : tC)+(t9_reg[15] ? -t9 : t9))*1+Tsub))|(sub_counter == (0+((tC_reg[15] ? -tC : tC)+(t9_reg[15] ? -t9 : t9))*1-Tsub))) )          
                            begin
                                out[9]=1;
                            end
                            
						if((manualEnable[9]==0) & ((sub_counter == 0+((tD_reg[15] ? -tD : tD)+(t9_reg[15] ? -t9 : t9))*1)|(sub_counter == (0+((tD_reg[15] ? -tD : tD)+(t9_reg[15] ? -t9 : t9))*1+Tsub))|(sub_counter == (0+((tD_reg[15] ? -tD : tD)+(t9_reg[15] ? -t9 : t9))*1-Tsub))) )          
                            begin
                                out[9]=0;
                            end
                            
						if((manualEnable[9]==0) & ((sub_counter == 0+((tE_reg[15] ? -tE : tE)+(t9_reg[15] ? -t9 : t9))*1)|(sub_counter == (0+((tE_reg[15] ? -tE : tE)+(t9_reg[15] ? -t9 : t9))*1+Tsub))|(sub_counter == (0+((tE_reg[15] ? -tE : tE)+(t9_reg[15] ? -t9 : t9))*1-Tsub))) )          
                            begin
                                out[9]=1;
                            end
                            
						if((manualEnable[9]==0) & ((sub_counter == 0+((tF_reg[15] ? -tF : tF)+(t9_reg[15] ? -t9 : t9))*1)|(sub_counter == (0+((tF_reg[15] ? -tF : tF)+(t9_reg[15] ? -t9 : t9))*1+Tsub))|(sub_counter == (0+((tF_reg[15] ? -tF : tF)+(t9_reg[15] ? -t9 : t9))*1-Tsub))) )          
                            begin
                                out[9]=0;
                            end
                            
                    // output 10, subsequence
						if((manualEnable[10]==0) & ((sub_counter == 0+((tC_reg[15] ? -tC : tC)+(t9_reg[15] ? -t9 : t9))*1)|(sub_counter == (0+((tC_reg[15] ? -tC : tC)+(t9_reg[15] ? -t9 : t9))*1+Tsub))|(sub_counter == (0+((tC_reg[15] ? -tC : tC)+(t9_reg[15] ? -t9 : t9))*1-Tsub))) )          
                            begin
                                out[10]=1;
                            end
                            
						if((manualEnable[10]==0) & ((sub_counter == 0+((tD_reg[15] ? -tD : tD)+(t9_reg[15] ? -t9 : t9))*1)|(sub_counter == (0+((tD_reg[15] ? -tD : tD)+(t9_reg[15] ? -t9 : t9))*1+Tsub))|(sub_counter == (0+((tD_reg[15] ? -tD : tD)+(t9_reg[15] ? -t9 : t9))*1-Tsub))) )          
                            begin
                                out[10]=0;
                            end
                            
						if((manualEnable[10]==0) & ((sub_counter == 0+((tE_reg[15] ? -tE : tE)+(t9_reg[15] ? -t9 : t9))*1)|(sub_counter == (0+((tE_reg[15] ? -tE : tE)+(t9_reg[15] ? -t9 : t9))*1+Tsub))|(sub_counter == (0+((tE_reg[15] ? -tE : tE)+(t9_reg[15] ? -t9 : t9))*1-Tsub))) )          
                            begin
                                out[10]=1;
                            end
                            
						if((manualEnable[10]==0) & ((sub_counter == 0+((tF_reg[15] ? -tF : tF)+(t9_reg[15] ? -t9 : t9))*1)|(sub_counter == (0+((tF_reg[15] ? -tF : tF)+(t9_reg[15] ? -t9 : t9))*1+Tsub))|(sub_counter == (0+((tF_reg[15] ? -tF : tF)+(t9_reg[15] ? -t9 : t9))*1-Tsub))) )          
                            begin
                                out[10]=0;
                            end
                            
                    // output 27, subsequence
						if((manualEnable[27]==0) & ((sub_counter == 0+((tA_reg[15] ? -tA : tA)+(t8_reg[15] ? -t8 : t8))*1)|(sub_counter == (0+((tA_reg[15] ? -tA : tA)+(t8_reg[15] ? -t8 : t8))*1+Tsub))|(sub_counter == (0+((tA_reg[15] ? -tA : tA)+(t8_reg[15] ? -t8 : t8))*1-Tsub))) )          
                            begin
                                out[27]=1;
                            end
                            
						if((manualEnable[27]==0) & ((sub_counter == 0+((tB_reg[15] ? -tB : tB)+(t8_reg[15] ? -t8 : t8))*1)|(sub_counter == (0+((tB_reg[15] ? -tB : tB)+(t8_reg[15] ? -t8 : t8))*1+Tsub))|(sub_counter == (0+((tB_reg[15] ? -tB : tB)+(t8_reg[15] ? -t8 : t8))*1-Tsub))) )          
                            begin
                                out[27]=0;
                            end
                            
                    // output 28, subsequence
						if((manualEnable[28]==0) & ((sub_counter == 0+((tC_reg[15] ? -tC : tC)+(t9_reg[15] ? -t9 : t9))*1)|(sub_counter == (0+((tC_reg[15] ? -tC : tC)+(t9_reg[15] ? -t9 : t9))*1+Tsub))|(sub_counter == (0+((tC_reg[15] ? -tC : tC)+(t9_reg[15] ? -t9 : t9))*1-Tsub))) )          
                            begin
                                out[28]=1;
                            end
                            
						if((manualEnable[28]==0) & ((sub_counter == 0+((tD_reg[15] ? -tD : tD)+(t9_reg[15] ? -t9 : t9))*1)|(sub_counter == (0+((tD_reg[15] ? -tD : tD)+(t9_reg[15] ? -t9 : t9))*1+Tsub))|(sub_counter == (0+((tD_reg[15] ? -tD : tD)+(t9_reg[15] ? -t9 : t9))*1-Tsub))) )          
                            begin
                                out[28]=0;
                            end
                            
						if((manualEnable[28]==0) & ((sub_counter == 0+((tE_reg[15] ? -tE : tE)+(t9_reg[15] ? -t9 : t9))*1)|(sub_counter == (0+((tE_reg[15] ? -tE : tE)+(t9_reg[15] ? -t9 : t9))*1+Tsub))|(sub_counter == (0+((tE_reg[15] ? -tE : tE)+(t9_reg[15] ? -t9 : t9))*1-Tsub))) )          
                            begin
                                out[28]=1;
                            end
                            
						if((manualEnable[28]==0) & ((sub_counter == 0+((tF_reg[15] ? -tF : tF)+(t9_reg[15] ? -t9 : t9))*1)|(sub_counter == (0+((tF_reg[15] ? -tF : tF)+(t9_reg[15] ? -t9 : t9))*1+Tsub))|(sub_counter == (0+((tF_reg[15] ? -tF : tF)+(t9_reg[15] ? -t9 : t9))*1-Tsub))) )          
                            begin
                                out[28]=0;
                            end
                            
                    // output 29, subsequence
						if((manualEnable[29]==0) & ((sub_counter == 0+((tA_reg[15] ? -tA : tA)+(t8_reg[15] ? -t8 : t8))*1)|(sub_counter == (0+((tA_reg[15] ? -tA : tA)+(t8_reg[15] ? -t8 : t8))*1+Tsub))|(sub_counter == (0+((tA_reg[15] ? -tA : tA)+(t8_reg[15] ? -t8 : t8))*1-Tsub))) )          
                            begin
                                out[29]=1;
                            end
                            
						if((manualEnable[29]==0) & ((sub_counter == 0+((tB_reg[15] ? -tB : tB)+(t8_reg[15] ? -t8 : t8))*1)|(sub_counter == (0+((tB_reg[15] ? -tB : tB)+(t8_reg[15] ? -t8 : t8))*1+Tsub))|(sub_counter == (0+((tB_reg[15] ? -tB : tB)+(t8_reg[15] ? -t8 : t8))*1-Tsub))) )          
                            begin
                                out[29]=0;
                            end
                            
                    // output 30, subsequence
						if((manualEnable[30]==0) & ((sub_counter == 0+((tA_reg[15] ? -tA : tA)+(t8_reg[15] ? -t8 : t8)+(tS_reg[15] ? -tS : tS))*1)|(sub_counter == (0+((tA_reg[15] ? -tA : tA)+(t8_reg[15] ? -t8 : t8)+(tS_reg[15] ? -tS : tS))*1+Tsub))|(sub_counter == (0+((tA_reg[15] ? -tA : tA)+(t8_reg[15] ? -t8 : t8)+(tS_reg[15] ? -tS : tS))*1-Tsub))) )          
                            begin
                                out[30]=1;
                            end
                            
						if((manualEnable[30]==0) & ((sub_counter == 0+((tB_reg[15] ? -tB : tB)+(t8_reg[15] ? -t8 : t8)+(tT_reg[15] ? -tT : tT))*1)|(sub_counter == (0+((tB_reg[15] ? -tB : tB)+(t8_reg[15] ? -t8 : t8)+(tT_reg[15] ? -tT : tT))*1+Tsub))|(sub_counter == (0+((tB_reg[15] ? -tB : tB)+(t8_reg[15] ? -t8 : t8)+(tT_reg[15] ? -tT : tT))*1-Tsub))) )          
                            begin
                                out[30]=0;
                            end
                            							
						sub_counter= sub_counter +1;
						if(sub_counter == Tsub)
							begin
								nsub = nsub + 1;
								sub_counter = 0;
								if(nsub == nsubmax)
									begin
										sub_running=0;
									out[6]=0; //Outputs unused in subseqs must be set to zero after end of subseq.
									out[7]=0; //Outputs unused in subseqs must be set to zero after end of subseq.
									out[8]=0; //Outputs unused in subseqs must be set to zero after end of subseq.
									out[9]=0; //Outputs unused in subseqs must be set to zero after end of subseq.
									out[10]=0; //Outputs unused in subseqs must be set to zero after end of subseq.
									out[27]=0; //Outputs unused in subseqs must be set to zero after end of subseq.
									out[28]=0; //Outputs unused in subseqs must be set to zero after end of subseq.
									out[29]=0; //Outputs unused in subseqs must be set to zero after end of subseq.
									out[30]=0; //Outputs unused in subseqs must be set to zero after end of subseq.

									end
							end
					end
				
				counter = counter + 1;
				
			end
    end

endmodule