#!/usr/bin/env python
import syncComSubseq as sync
import shutil
import pprint

class Synchronizer:
    """ An interface for the synchronizer, tested with Digilent Nexys 3 FPGA. """
    ''' Usage:
    Modify Self.SyncCfg to chose your config. Can be defined in an external .synccfg file, the compiled filed can be saved along with such a file.
    Compile, Load and Start when ready.
    '''
    def __init__(self):
        default_cfg={
            'period':(0,60,0,0),
            'outputs':[
                {'description':'gradB',             'bnc_name':'DIO7',  'sequence':[(0,0,0,0),(0,53,0,0)],          'subseq':[],    'manual_enable':0,'manual_value':0},
                {'description':'MOTbeams',          'bnc_name':'AO1',   'sequence':[(0,0,0,0),(0,53,0,0)],          'subseq':[],    'manual_enable':0,'manual_value':0},
                {'description':'MOTrepump',         'bnc_name':'AO2',   'sequence':[(0,0,0,0),(0,53,'t1',0)],       'subseq':[],    'manual_enable':0,'manual_value':0},
                {'description':'probe 4-5',         'bnc_name':'AO3',   'sequence':[(0,52,0,0),(0,52,10,0)],'subseq':[(0,0,0,10),(0,0,1,0)],    'manual_enable':0,'manual_value':0},
                {'description':'probe 4-4 matisse', 'bnc_name':'DIO0',  'sequence':[(0,53,'t2',0),(0,53,'t2+t3',0)],'subseq':[],    'manual_enable':1,'manual_value':0},
                {'description':'control 3-4 casper','bnc_name':'DIO1',  'sequence':[(0,53,'t4',0),(0,53,'t4+t5',0)],'subseq':[],    'manual_enable':1,'manual_value':0},
                {'description':'TestScopeCH2',      'bnc_name':'AI2+',  'sequence':[(0,53,0,0),(0,'53+t6',0,0)],    'subseq':[],    'manual_enable':0,'manual_value':0},
                {'description':'TestScopeCH4',      'bnc_name':'AI4+',  'sequence':[(0,53,0,0),(0,'53+t7',0,0)],    'subseq':[],    'manual_enable':0,'manual_value':0},
             ],
             'tparams':[0,240,0,1,240,20,20,50,0,0,0,0,0,0,0,0,0,240,0,1,240,20,20,50,0,0,0,0,0,0,0,0],
             'subsequence':{'tsub':53000,'nsubmax':100,'Tsub':1000}, #for subsequence: tsub, subsequence starting time in musec, nsubmax: number of subsequences, Tsub duration of a subsequence in 10ns unit
             'autotrig':True,
             'compiled':False,
             'filepath':''
        }

        self.SyncCfg=default_cfg        
        self.loaded=False
        self.running=False

    def outputs_listkey(self,key):
        """ Returns a list of outputs values for a given key. Used by other methods of this class. The key may be for example, for a given output, its 'sequence' or its 'bnc_name'. """
        return [output[key] for output in self.SyncCfg['outputs']]

    def compCfg(self,nOutputs=-1):
        """
        Produces a .bit and .xsvf specific config file for the fpga, from the argument SyncCfg (user defined synchronizing configuration)

        Times are all in the 4-tuple format (sec,millisec,microsec,nanosec).
        They may be numbers or strings expect for nanoseconds (numbers only). Strings may contain 'n' (the number of cycles already played), 't0', 't1'... 't9': Parameters that can be set on demand when the configuration is already loaded, and mathematic operations such as addition/substraction.

        A synchronizer config is a dictionary with:
            'period' : (some,timing,0,0)
            'outputs': [list of outputs], each output is a dictionary
                {'description':'some description','bnc_name':'DIO0','sequence':[(time1,0,0,0),(time2,0,0,0),...],'manual_enable':False,'manual_value':False}
            'tparams': [1,20,45,....]
            'autotrig': 1

        'manual_enable' allows for manual definition of the output value : when set to True the sequence doesn't play a role anymore, instead manual_value is used continuously.
        'autotrig' allows continuous and internal trigger of the sequence, when set to False the sequence can be trigged by and external trigger signal.

        nOutputs argument allows to compile only first nOutputs outputs of SyncCfg. Default, -1 := all outputs.   
        """
        if nOutputs==-1:
            nOutputs = len(self.SyncCfg['outputs'])
            
        if not self.SyncCfg['compiled']:
            #Compile: Writes the necessary verilog and .ucf file, then compile. A .xsvf file is generated.
            sublist=self.outputs_listkey('subseq')[:nOutputs]
            
            sync.prepare(self.SyncCfg['period'],self.outputs_listkey('sequence')[:nOutputs],self.outputs_listkey('bnc_name')[:nOutputs],sublist)
            self.SyncCfg['compiled']=True
            self.SyncCfg['filepath']=sync.sync_files()+'/sync.xsvf'
            self.loaded=False
        else:
            print 'Config already compiled.'
        
    def loadCfg(self):
        """
        Loads the .xsvf specific config file for the fpga to the fpga board.
        """
        if not self.SyncCfg['compiled']:
            print "Warning, the current config has not been compiled yet. Loading an older confg."
        sync.main.bit2Fpga(self.SyncCfg['filepath'])

        self.updateparams()
        
        self.loaded=True        
        self.running=False

    def updateparams(self):
        fpga_handle=sync.getFpgaHandle()
        
        self.updateManualEnable(fpga_handle)
        self.updateManualValues(fpga_handle)
        self.updateAutotrig(fpga_handle)
        self.updateTparams(fpga_handle)
        self.updateSubsequence(fpga_handle)

        sync.closeFpgaHandle(fpga_handle)
        
    def updateManualEnable(self,fpga_handle=None):
        sync.manualEnable(self.outputs_listkey('manual_enable'),fpga_handle)
        
    def updateManualValues(self,fpga_handle=None):
        sync.manualValues(self.outputs_listkey('manual_value'),fpga_handle)
        
    def updateAutotrig(self,fpga_handle=None):
        sync.autotrig(self.SyncCfg['autotrig'],fpga_handle)
        
    def updateTparams(self,fpga_handle=None):
        if fpga_handle==None:
            fpga_handle=sync.getFpgaHandle()
            to_be_closed=True
        else:
            to_be_closed=False
        
        for i in range(len(self.SyncCfg['tparams'])):
            sync.redefine_tparams(i,self.SyncCfg['tparams'][i],fpga_handle)

        if to_be_closed:
            sync.closeFpgaHandle(fpga_handle)
		
    def updateTparam(self,i,fpga_handle=None):
        sync.redefine_tparams(i,self.SyncCfg['tparams'][i],fpga_handle)

    def updateSubsequence(self,fpga_handle=None):
        p = self.SyncCfg['subsequence']
        sync.redefine_subsequence(p['tsub'],p['nsubmax'],p['Tsub'],fpga_handle)

    def start(self,fpga_handle=None):
        sync.start(fpga_handle)
        self.running=True

    def stop(self,fpga_handle=None):
        sync.stop(fpga_handle)
        self.running=False
        
    def savetofile(self,path):
        ''' Save config in a .cfg file and save compiled file. The path given should be in the form path/to/file/filename.cfg (with .cfg extension). The .cfg file produced will not be as easy to read as a file written manually.'''
        #Copy compiled file, change file path in config
        if self.SyncCfg['compiled']:
            cpath=path[:-4]+'.xsvf'
            if self.SyncCfg['filepath'] != cpath:
                shutil.copyfile(self.SyncCfg['filepath'],cpath)
            self.SyncCfg['filepath']=cpath     
        
        f = open(path, 'w')
        if(pprint.isreadable(self.SyncCfg)==False):
            print "synchronizer.savetofile() WARNING: .cfg file might not be readable with eval (pprint.isreadable() returned false)"
        f.write( 'self.SyncCfg = ' + pprint.pformat(self.SyncCfg,indent=4) + '\n' )
        f.close()

    def saveCompdFile(self,path):
        ''' Only save the compiled file. Usefull if the .cfg was defined from a text editor in a more readable way than the files produces by savetofile() method. '''
        if self.SyncCfg['compiled']:
            cpath=path[:-4]+'.xsvf'
            shutil.copyfile(self.SyncCfg['filepath'],cpath)
            self.SyncCfg['filepath']=cpath
        else:
            print 'synchronizer.py, saveCompdFile: The current SyncCfg was not compiled yet. '           

    def loadfromfile(self,path):
        exec(open(path,'r').read())
        self.loaded=False

    def findbydescription(self,desc):
        for elem in self.SyncCfg['outputs']:
            if elem['description']==desc:
                return elem
        print 'synchronizer.py findbydescription: The given output description (%s) was not found in the current SyncCfg'%desc
        return None
        
if __name__ =="__main__":
    print('From synchronizer.py')
    s=Synchronizer()
    #s.loadfromfile('SyncConfigs/default.cfg')
    #s.compCfg()
    #s.savetofile('SyncConfigs/saved.cfg')
    
    #s.loadfromfile('SyncConfigs/saved.cfg')
    #s.loadCfg()
    #s.start()
    #s.SyncCfg['outputs'][4]['manual_enable']=0
    #s.SyncCfg['outputs'][5]['manual_enable']=0
    #s.updateManualEnable()

